#!perl -w
use strict;

BEGIN {
    if ($ENV{LANG} ne 'C') {
        system("LANG=C $ENV{PERL} $0");
        exit;
    }
}

use Test::More tests => 48;
use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::Core::API::File qw(ExistsFile DeleteFile WriteFile
                                  GetFileContent GetFileContentLines);
use DE_EPAGES::Core::API::SystemCommand qw(SystemCommand);
use DE_EPAGES::Core::API::Script qw(RunScript);

RunScript( Message => 'long', Sub => sub {

    my $TempFile  = 'file.tmp';
    my $Content = "Line 1\nLine 2";
    my $got;

    # ------------------------------------------------------------------------
    # write with lz4 compression
    # ------------------------------------------------------------------------
    WriteFile( $TempFile, \$Content, {compressed => 'lz4'} );
    $got = SystemCommand("file $TempFile", {Capture => 1});
    print STDERR "\n$got";
    like($got, qr/data/, 'file -> data');
    $got = GetFileContent($TempFile, {compressed => 'lz4'});
    is($$got, $Content, 'GetFileContent (is compressed)');
    $got = GetFileContent($TempFile);
    isnt($$got, $Content, 'GetFileContent (isnt uncompressed)');

    my $aLines = GetFileContentLines($TempFile, {compressed => 'lz4'});
    is( $aLines->[0], "Line 1\n", 'GetFileContentLines 1' );
    is( $aLines->[1], 'Line 2', 'GetFileContentLines 2' );

    WriteFile( $TempFile, \$Content );
    $got = SystemCommand("file $TempFile", {Capture => 1});
    print STDERR $got;
    like($got, qr/ASCII text/, 'file -> ASCII text');
    is(${GetFileContent($TempFile)}, $Content, 'GetFileContent (is uncompressed)');

    eval { GetFileContent($TempFile, {compressed => 'lz4'}); };
    ok(ExistsError(), 'GetFileContent -> ExistsError');
    is(GetError()->code, 'LZ4_UNCOMPRESS_FAILED', 'GetError -> LZ4_UNCOMPRESS_FAILED');

    eval { GetFileContent('foobar', {compressed => 'lz4'}); };
    ok(ExistsError(), 'GetFileContent -> ExistsError');
    is(GetError()->code, 'FILE_NOT_FOUND', 'GetError -> FILE_NOT_FOUND');

    # ------------------------------------------------------------------------
    # write with zstd compression
    # ------------------------------------------------------------------------
    WriteFile( $TempFile, \$Content, {compressed => 'zstd'} );
    $got = SystemCommand("file $TempFile", {Capture => 1});
    print STDERR $got;
    like($got, qr/data/, 'file -> data');
    $got = GetFileContent($TempFile, {compressed => 'zstd'});
    is($$got, $Content, 'GetFileContent (is compressed)');
    $got = GetFileContent($TempFile);
    isnt($$got, $Content, 'GetFileContent (isnt uncompressed)');

    $aLines = GetFileContentLines($TempFile, {compressed => 'zstd'});
    is( $aLines->[0], "Line 1\n", 'GetFileContentLines 1' );
    is( $aLines->[1], 'Line 2', 'GetFileContentLines 2' );

    WriteFile( $TempFile, \$Content );
    $got = SystemCommand("file $TempFile", {Capture => 1});
    print STDERR $got;
    like($got, qr/ASCII text/, 'file -> ASCII text');
    is(${GetFileContent($TempFile)}, $Content, 'GetFileContent (is uncompressed)');

    eval { GetFileContent($TempFile, {compressed => 'zstd'}); };
    ok(ExistsError(), 'GetFileContent -> ExistsError');
    is(GetError()->code, 'ZSTD_UNCOMPRESS_FAILED', 'GetError -> ZSTD_UNCOMPRESS_FAILED');

    # ------------------------------------------------------------------------
    # write with gzip compression
    # ------------------------------------------------------------------------
    WriteFile( $TempFile, \$Content, {compressed => 'gzip'} );
    $got = SystemCommand("file $TempFile", {Capture => 1});
    print STDERR $got;
    like($got, qr/gzip compressed data/, 'file -> gzip compressed data');
    is(${GetFileContent($TempFile, {compressed => 'gzip'})}, $Content,
       'GetFileContent (is compressed)');
    isnt(${GetFileContent($TempFile)}, $Content, 'GetFileContent (isnt uncompressed)');

    $aLines = GetFileContentLines($TempFile, {compressed => 'gzip'});
    is( $aLines->[0], "Line 1\n", 'GetFileContentLines 1' );
    is( $aLines->[1], 'Line 2', 'GetFileContentLines 2' );

    WriteFile( $TempFile, \$Content );
    $got = SystemCommand("file $TempFile", {Capture => 1});
    print STDERR $got;
    like($got, qr/ASCII text/, 'file -> ASCII text');
    is(${GetFileContent($TempFile)}, $Content, 'GetFileContent (is uncompressed)');

    # uncompressed file can be read with {compressed => 'gzip'} !
    is(${GetFileContent($TempFile, {compressed => 'gzip'})}, $Content,
       'GetFileContent (compressed)');

    # ------------------------------------------------------------------------
    # test utf8
    # ------------------------------------------------------------------------
    $Content = "Line \x{100}a\x{101}\nLine 2";
    WriteFile( $TempFile, \$Content, {utf8 => 1} );
    $got = SystemCommand("file $TempFile", {Capture => 1});
    print STDERR $got;
    like($got, qr/Unicode text/, 'file -> Unicode text');
    $aLines = GetFileContentLines($TempFile, {utf8 => 1});
    is( $aLines->[0], "Line \x{100}a\x{101}\n", 'GetFileContentLines 1' );
    is( $aLines->[1], 'Line 2', 'GetFileContentLines 2' );
    $got = GetFileContent($TempFile, {utf8 => 1});
    is($$got, $Content, 'GetFileContent($TempFile, {utf8 => 1})');

    WriteFile( $TempFile, \$Content, {utf8 => 1, compressed => 'gzip'} );
    $got = SystemCommand("file $TempFile", {Capture => 1});
    print STDERR $got;
    like($got, qr/gzip compressed data/, 'file -> gzip compressed data');
    $aLines = GetFileContentLines($TempFile, {utf8 => 1, compressed => 'gzip'});
    is( $aLines->[0], "Line \x{100}a\x{101}\n", 'GetFileContentLines 1' );
    is( $aLines->[1], 'Line 2', 'GetFileContentLines 2' );
    $aLines = GetFileContentLines($TempFile, {compressed => 'gzip'});
    isnt( $aLines->[0], "Line \x{100}a\x{101}\n", 'isnt GetFileContentLines' );
    $got = GetFileContent($TempFile, {utf8 => 1, compressed => 'gzip'});
    is($$got, $Content, 'GetFileContent($TempFile, {utf8 => 1, compressed => gzip})');

    WriteFile( $TempFile, \$Content, {utf8 => 1, compressed => 'zstd'} );
    $got = SystemCommand("file $TempFile", {Capture => 1});
    print STDERR $got;
    like($got, qr/data/, 'file -> data');
    $aLines = GetFileContentLines($TempFile, {utf8 => 1, compressed => 'zstd'});
    is( $aLines->[0], "Line \x{100}a\x{101}\n", 'GetFileContentLines 1' );
    is( $aLines->[1], 'Line 2', 'GetFileContentLines 2' );
    $aLines = GetFileContentLines($TempFile, {compressed => 'zstd'});
    isnt( $aLines->[0], "Line \x{100}a\x{101}\n", 'isnt GetFileContentLines' );
    $got = GetFileContent($TempFile, {utf8 => 1, compressed => 'zstd'});
    is($$got, $Content, 'GetFileContent($TempFile, {utf8 => 1, compressed => zstd})');

    WriteFile( $TempFile, \$Content, {utf8 => 1, compressed => 'lz4'} );
    $got = SystemCommand("file $TempFile", {Capture => 1});
    print STDERR $got;
    like($got, qr/data/, 'file -> data');
    $aLines = GetFileContentLines($TempFile, {utf8 => 1, compressed => 'lz4'});
    is( $aLines->[0], "Line \x{100}a\x{101}\n", 'GetFileContentLines 1' );
    is( $aLines->[1], 'Line 2', 'GetFileContentLines 2' );
    $aLines = GetFileContentLines($TempFile, {compressed => 'lz4'});
    isnt( $aLines->[0], "Line \x{100}a\x{101}\n", 'isnt GetFileContentLines' );
    $got = GetFileContent($TempFile, {utf8 => 1, compressed => 'lz4'});
    is($$got, $Content, 'GetFileContent($TempFile, {utf8 => 1, compressed => lz4})');

    DeleteFile($TempFile);
    ok(!ExistsFile($TempFile), 'DeleteFile');
});
