﻿#========================================================================================
# §package      DE_EPAGES::Object::API::TLE::StandardHandler
# §state        public
#----------------------------------------------------------------------------------------
# §description  Implements general TLE variables
#               <ul>
#                   <li>Currency - information about the currently selected currency</li>
#                   <li>System - root object of the current database</li>
#                   <li>StoreRoot - public image directory of the current database</li>
#               </ul>
#               Implements the format [webpath] for public file attributes.
#----------------------------------------------------------------------------------------
# §example      #Currency.Symbol  -> $
#               #System.StoreName -> Store
#               #System.StoreRoot        -> /WebRoot/Store
#               <img src="#ImageSmall[webpath]" />
#========================================================================================
package DE_EPAGES::Object::API::TLE::StandardHandler;

use strict;
use utf8;
use DE_EPAGES::Core::API::Error qw( Error );
use DE_EPAGES::Core::API::Log qw( GetLog );
use DE_EPAGES::Core::API::Warning qw ( Warning );
use DE_EPAGES::Object::API::Factory qw (LoadRootObject);
use DE_EPAGES::Object::API::Language;
use DE_EPAGES::Object::API::Table::Currency qw( GetInfoCurrency );

sub ErrorDefinition {
    return {
        ObsoleteTLE => '#TemplateName: Usage of global TLE "#StoreRoot" is deprecated, use "#System.StoreRoot" instead!'
    };
}

#========================================================================================
# §function     RegisterHandlerProc
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $RegisterHandlerProc( $hParams );
#----------------------------------------------------------------------------------------
# §description  this function registers this attribute handle on the processor via hook
#               function on hook TLEProcessorRegistration
#----------------------------------------------------------------------------------------
# §input        $hParams | hook arguments keys :
#               <ul>
#                   <li>Processor - tle processor - object
#               </ul> | ref.hash
#========================================================================================
sub RegisterHandlerProc {
    my ($Params) = @_;
    GetLog->debug( 'StandardHandler.RegisterHandlerProc' );

    __PACKAGE__->new()->register( $Params->{'Processor'} );
    return;
}

#========================================================================================
# §function     new
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Handler = new DE_EPAGES::Object::API::TLE::StandardHandler;
# §example      my $PrefixHandler  = new DE_EPAGES::Object::API::TLE::StandardHandler;
#----------------------------------------------------------------------------------------
# §description  create a new handler
#----------------------------------------------------------------------------------------
# §return       $Handler | blessed object | object
#========================================================================================
sub new { bless {}, shift; }

#========================================================================================
# §function     register
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Handler->register( $Processor );
#----------------------------------------------------------------------------------------
# §description  this function register this handle on the processor
#----------------------------------------------------------------------------------------
# §input        $Processor | tle processor | object
#========================================================================================
sub register {
    my $self = shift;
    my ($Processor) = @_;
    GetLog->debug( 'StandardHandler.register' );

    $Processor->registerHandler('VariableHandler', $self, 'Currency');
    $Processor->registerHandler('VariableHandler', $self, 'System');
    $Processor->registerHandler('VariableHandler', $self, 'StoreRoot');
    $Processor->registerHandler('VariableHandler', $self, 'StaticResourceURL');
    $Processor->registerHandler('FunctionHandler', $self, 'WITH_LANGUAGE');
    $Processor->registerHandler('FunctionHandler', $self, 'CURRENCY');
    $Processor->registerHandler('FormatHandler',   $self, 'webpath');
    return;
}

#========================================================================================
# §function     tle
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Value = $Handler->tle( $Processor, $TLEName);
# §example      my $Value = $Handler->tle( $Processor, 'Name');
#----------------------------------------------------------------------------------------
# §description  This function is called by tle processor to get the value of a tle
#               variable.
#----------------------------------------------------------------------------------------
# §input        $Processor | tle processor | object
# §input        $TLEName | tle name | string
# §return       $Value | return value | integer
#========================================================================================
sub tle {
    my $self = shift;
    my ($Processor, $TLEName) = @_;
    GetLog->debug( "StandardHandler.tle($TLEName)" );

    if( $TLEName eq 'System' ) {
        return LoadRootObject()->tleHash;
    }
    elsif( $TLEName eq 'StoreRoot' ) {
        # show deprecated warning
        Warning('ObsoleteTLE',
          {
              TemplateName => $Processor->getContext->{'TemplateFileName'}
          });
        return LoadRootObject()->get('StoreRoot');
    }
    elsif( $TLEName eq 'Currency' ) {
        my $CurrencyID = $Processor->tle('CurrencyID');
        $CurrencyID = $Processor->tle('INPUT.Currency') unless defined $CurrencyID;
        return GetInfoCurrency($CurrencyID);
    }

    return undef;
}

#========================================================================================
# §function     existsTLE
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Value = $Handler->existsTLE( $Processor, $TLEName);
# §example      my $Value = $Handler->existsTLE( $Processor, 'Products_Count');
#----------------------------------------------------------------------------------------
# §description  This function is called by the tle processor to test if a TLE variable
#               is available.
#----------------------------------------------------------------------------------------
# §input        $Processor | tle processor | object
# §input        $TLEName | tle name | string
# §return       $Value | yes or no | bool
#========================================================================================
sub existsTLE {
    my $self = shift;
    my ($Processor, $TLEName) = @_;
    GetLog->debug( "StandardHandler.existsTLE($TLEName)" );

    return (defined $self->tle($Processor, $TLEName)) ? 1 : 0;
}

#========================================================================================
# §function     WITH_LANGUAGE
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       #WITH_LANGUAGE(#LanguageID) ... #ENDWITH_LANGUAGE
# §example      #LOCAL("LocalLanguage", #FUNCTION("GetLanguageByCode", "en"))
#                 #WITH_LANGUAGE(#LocalLanguage.LanguageID)
#                   <h3>#NameOrAlias (#LocalLanguage.Code3)</h3>
#                   <p>#Description</p>
#                 #ENDWITH_LANGUAGE
#               #ENDLOCAL
#----------------------------------------------------------------------------------------
# §description  Temporarily changes the language context of a template.
#----------------------------------------------------------------------------------------
# §input        #LanguageID | language id | integer
#========================================================================================
sub WITH_LANGUAGE { ## no critic qw(RequireCommentForExportedFunctions)
    my $self = shift;
    my ($Processor, $raParams, $cTemplate) = @_;
    Error('NoLanguageID', {'LanguageID' => $raParams->[0]}) unless $raParams->[0] =~ /^\d+$/;
    local $DE_EPAGES::Object::API::Language::LANGUAGEID = $raParams->[0];
    return join('', @{$Processor->replaceTLEs($cTemplate)});
}

#========================================================================================
# §function     CURRENCY
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       #CURRENCY(#CurrencyID)
# §example      #LOCAL("LocalCurrency", #CURRENCY("USD"))
#                 #LocalCurrency.Symbol
#               #ENDLOCAL
#----------------------------------------------------------------------------------------
# §description  Returns currency information by curreny id.
#----------------------------------------------------------------------------------------
# §return       #Currency | currency information with following keys:
#               <ul>
#                   <li>CurrencyID    - alpha currency code (ISO 4217) (primary key)- char(3)
#                   <li>IsUsed        - is currency used     - bit
#                   <li>NumCode       - numeric code of currency iso 4217 - int
#                   <li>Name          - name of currency     - varchar(255)
#                   <li>MinorUnit     - decimals of minor unit (optional)- int
#                   <li>LeadCurrencyID - lead currency code   (optional)- char(3)
#                   <li>Factor        - factor lead to current (Currency = Lead * Factor)        (optional)- numeric(20,10)
#                   <li>Symbol        - default symbol of currency (optional)- nvarchar(5)
#                   <li>SymbolOrNumCode - default symbol of currency or alpha currency code if symbol is not exists - char
#                   <li>PosFormat     - format for positive numbers, symbol in front of or behind number ('¤ x' or 'x ¤'), x will be replaced by number ¤ by currency symbol. (optional)- varchar(10)
#                   <li>NegFormat     - format for negative numbers, symbol in front of or behind number ('-¤ x' or 'x ¤-'), x will be replaced by number ¤ by currency symbol. its only allowed to use '-' for negative numbers. (optional)- varchar(10)
#               </ul>| reference to hash
#========================================================================================
sub CURRENCY { ## no critic qw(RequireCommentForExportedFunctions)
    my $self = shift;
    my ($Processor, $raParams, $cTemplate) = @_;
    Error('NoCurrencyID', {'CurrencyID' => $raParams->[0]}) unless defined $raParams->[0];

    my $hInfoCurrency = GetInfoCurrency($raParams->[0]);

    # add additional key for having a symbol or the alpha currency code
    $hInfoCurrency->{'SymbolOrNumCode'} = $hInfoCurrency->{'Symbol'};
    $hInfoCurrency->{'SymbolOrNumCode'} = $hInfoCurrency->{'CurrencyID'} unless defined $hInfoCurrency->{'SymbolOrNumCode'};

    return $hInfoCurrency;
}


#========================================================================================
# §function     Format
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Value = $Handler->Format($Processor, $Format, $Value, $TLEName);
# §example      my $Value = $Handler->Format($Processor, $Format, $Value, $TLEName);
#----------------------------------------------------------------------------------------
# §description  this formatter decides if the WebPath of object will be added to file name
#               or not. this depends on the value, if the hostname exists (//) in value
#               the WebPath will not be added.
#----------------------------------------------------------------------------------------
# §input        $Processor | tle processor | object
# §input        $Format | 'webpath' | string
# §input        $Value | unformatted file url | string
# §input        $TLEName | tle name of value (not used) | string
# §return       $Value | formatted value | string
#========================================================================================
sub Format {
    my $self = shift;
    my ($Processor, $Format, $Value, $TLEName) = @_;
    return undef unless $Format eq 'webpath';
    return undef unless defined $Value;
    return $Value if $Value =~ /^\w+\:\/\//; # starts with protocol://hostname
    return $Processor->tle('Site')->{'WebPath'} . '/'.$Value if $Value =~ /^MediaGallery[\/\\]/; # starts MediaGallery/
    my $WebPath = $Processor->tle('WebPath');
    return $Value if $Value =~ /$WebPath/gi;
    return $Value if $Value =~ /^\/storage\/images/gi;
    return $WebPath.'/'.$Value;
}

1;
