use strict;
use DateTime::Format::ISO8601;
use JSON qw( from_json );
use Getopt::Long;
use Time::HiRes qw (time);
use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::Warning qw( Warning );
use DE_EPAGES::Database::API::Connection qw( RunOnStore );
use DE_EPAGES::Object::API::Factory qw( LoadObjectByPath ExistsObjectByPath LoadObject ExistsObject );
use DE_EPAGES::Object::API::Language qw( GetPKeyLanguageByCode );
use DE_EPAGES::WebInterface::API::MessageCenter qw( SynchronizeCache );

## no critic qw( ProhibitExcessComplexity )
sub setAttributes {
    my ( $Path, $ObjectID, $hAttributes, $LanguageCode, $Output ) = @_;

    my $Object;
    if($Path){
        if (!ExistsObjectByPath($Path)) {
            print STDERR "\nERROR: Object does not exist: '$Path'.\n\n";
            return -1;
        }
        $Object = LoadObjectByPath($Path);
    } elsif ($ObjectID){
        if (!ExistsObject($ObjectID)) {
            print STDERR "\nERROR: Object does not exist: '$ObjectID'.\n\n";
            return -1;
        }
        $Object = LoadObject($ObjectID);
    }
    my $LanguageID = defined $LanguageCode
        ? GetPKeyLanguageByCode($LanguageCode)
        : undef;

    my $Class = $Object->class();
    my $AllChecksOk = 1;

    # Check every attribute before trying to set it
    foreach my $Alias ( keys %$hAttributes ) {
        if (not $Class->existsAttribute( $Alias )) {
            print STDERR "\nERROR: Attribute '$Alias' does not exist on '$Path'.\n\n";
            $AllChecksOk = 0;
        }
        my $hInfoAttribute = $Class->attribute($Alias)->get([qw(Type TypeClass IsObject IsArray IsReadOnly)]);
        if ($hInfoAttribute->{'IsReadOnly'}) {
            print STDERR "\nERROR: Attribute $Alias is readonly on '$Path'.\n";
            $AllChecksOk = 0;
        }
        if ($hInfoAttribute->{'IsArray'}) {
            print STDERR "\nERROR: Cannot set Array Attribute '$Alias' on '$Path'\n";
            $AllChecksOk = 0;
        }
        if ($hInfoAttribute->{'IsObject'}) {
            if ($hAttributes->{$Alias} ne 'undef' and not ExistsObjectByPath($hAttributes->{$Alias})) {
                print STDERR "\nERROR: Cannot set object-valued Attribute $Alias on $Path to '".$hAttributes->{$Alias}.
                        "' because target object does not exist\n";
                $AllChecksOk = 0;
            } else {
                if ($hAttributes->{$Alias} ne 'undef') {
                    my $AttributeClass = $hInfoAttribute->{'TypeClass'};
                    my $TargetObject = LoadObjectByPath($hAttributes->{$Alias});
                    my $TargetClass = $TargetObject->class;
                    if ( not $AttributeClass->isSubClassOf( $TargetClass ) ) {
                        print STDERR "\n".'ERROR: Cannot set Attribute '.$Alias. ' on '. $Path.
                                ' because type mismatch [Attribute Type=' .$hInfoAttribute->{'Type'}.
                                ', Target Type='. $TargetClass->alias.']'."\n";
                        $AllChecksOk = 0;
                    }
                }
            }
        }
        if ($hInfoAttribute->{'Type'} =~ /Localized/ and not defined $LanguageID) {
            print STDERR "\nERROR: Attribute '".$Alias."' on '$Path' is of a localized type (".$hInfoAttribute->{'Type'}." but no -lang parameter was given!\n";
            $AllChecksOk = 0;
        }
    }

    if ( $AllChecksOk ) {
        my %SetAttributes = ();
        foreach my $Alias (keys %$hAttributes) {
            my $hInfoAttribute = $Class->attribute($Alias)->get([qw(Type IsObject)]);
            $Class = $Object->class;
            my $Attribute = $Class->attribute($Alias);
            if ($hAttributes->{$Alias} eq 'undef') {
                printf("Unset '%s'\n", $Alias, $hAttributes->{$Alias}) if $Output;
                $SetAttributes{$Alias} = undef;
            } else {
                if ($hInfoAttribute->{'Type'} =~ /Localized/ and defined $LanguageID) {
                    printf("set '%s'%s => '%s'\n", $Alias, $LanguageID?'['.$LanguageCode.']':'', $hAttributes->{$Alias}) if $Output;
                    $SetAttributes{$Alias} = $hAttributes->{$Alias};
                } elsif( $Attribute->isObject && ! $Attribute->isArray ) {
                    printf("set '%s'=> '%s' (%s)\n", $Alias, $hAttributes->{$Alias}, $hInfoAttribute->{'Type'} ) if $Output;
                    my $TargetObject = LoadObjectByPath($hAttributes->{$Alias});
                    $SetAttributes{$Alias} = $TargetObject;
                } elsif( $Attribute->type eq 'JSON' ) {
                    printf("set '%s' => '%s'\n", $Alias, $hAttributes->{$Alias}) if $Output;
                    $SetAttributes{$Alias} =  from_json( $hAttributes->{$Alias});
                } elsif( $Attribute->type eq 'DateTime' ) {
                    my $Formatter = DateTime::Format::ISO8601->new;
                    printf("set '%s' => '%s'\n", $Alias, $hAttributes->{$Alias}) if $Output;
                    $SetAttributes{$Alias} = $Formatter->parse_datetime($hAttributes->{$Alias});
                } else {
                    printf("set '%s' => '%s'\n", $Alias, $hAttributes->{$Alias}) if $Output;
                    $SetAttributes{$Alias} = $hAttributes->{$Alias};
                }
            }
        }
        if( defined $LanguageID ) {
            $Object->set( \%SetAttributes, $LanguageID );
        } else {
            $Object->set( \%SetAttributes );
        }
        return 0;
    } else {
        return -1;
    }
}
## use critic

sub Main {

    local $| = 1;

    my $Password;
    my $StoreName;
    my $Path;
    my $ObjectID;
    my $LanguageCode;
    my $Help;
    my $Output = 1;

    GetOptions(
        'help'          => \$Help,
        'passwd=s'      => \$Password,
        'storename=s'   => \$StoreName,
        'path=s'        => \$Path,
        'objectid=s'    => \$ObjectID,
        'lang=s'        => \$LanguageCode,
        'output!'       => \$Output,
    );

    usage() if $Help;
    unless($StoreName) {
        print STDERR "Missing parameter -storename\n\n";
        usage();
    }
    if (! defined $Path && ! defined $ObjectID) {
        print STDERR "Missing parameter -path or -objectid \n\n";
        usage();
    }
    my %Attributes = map { /(.+?)=(.*)/ ? ($1 => $2) : () } @ARGV;
    unless(keys %Attributes) {
        print STDERR "\nNo attribute/value pairs specified\n\n";
        usage();
    }

    my $time1 = time;
    RunOnStore(
        Store => $StoreName,
        DBPassword => $Password,
        Sub => sub  {
            my $result = setAttributes( $Path,$ObjectID, \%Attributes, $LanguageCode, $Output);
            if ($result < 0) { usage(); }
        },
    );
    my $time2 = time;
    printf "need %.3f seconds\n", $time2-$time1;

    eval { SynchronizeCache(); };
    Warning( GetError() ) if ExistsError();
}

sub usage {
    print <<END_USAGE;
Description:

Set non-array attributes of any object to a scalar or object. Note that the value
must be the full object path of an existing object in case the attribute is
an object and types must also be compatible. Using the special value 'undef' unsets
the value

Usage:
    perl $0 [options] [flags] [storename] attribute=value ...

options:
    -passwd     database user password (optional)
    -storename  name of store (uses [storename] if not specified)
    -lang       ISO language code (optional)
    -path       path to object (required, defaults to "/")
    -objectid   identifier of object (required or -path)
    -nooutput   suppress non-error output
flags:
    -help       show the command line options

Example:
    perl $0 -st Store -path "/" -lang en UPSSenderCity="Return To/Ship For City or Town"
    perl $0 -st Store -path "/" TaxMatrix=/TaxMatrixGermany
    perl $0 -st Store -path "/" TPay_logging=1
    perl $0 -st Store -path "/" ContractNumber=undef
    perl $0 -st Store -path "/Shops/DemoShop" NonMemberCustomerGroup=/Shops/DemoShop/Groups/NewCustomer

END_USAGE
    exit 2;
}

RunScript(
    'Sub' => \&Main,
    'Message' => 'long',
);
