#========================================================================================
# package      DE_EPAGES::Order::Hooks::OrderProcess
# state        public
# description  Hook functions for basket to order process.
#========================================================================================
package DE_EPAGES::Order::Hooks::OrderProcess;

use strict;

#========================================================================================
# function     OnRegisterCheckoutProcess
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnRegisterCheckoutProcess( $hParams );
#----------------------------------------------------------------------------------------
# description  UI_RegisterCheckoutProcess triggered, to register the basket step as a step for the
#               order process and for setting the order for the order process steps
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>BasketOrderProcess
#                   <li>Servlet
#               </ul> | ref.hash.*
#========================================================================================
sub OnRegisterCheckoutProcess {
    my ($hParams) = @_;

    my $BasketOrderProcess = $hParams->{'BasketOrderProcess'};


    # register all standard steps for showing in the order process
    $BasketOrderProcess->addStep('Basket','1000');
    $BasketOrderProcess->addStep('Address','2000');
    $BasketOrderProcess->addStep('Shipping','3000');
    $BasketOrderProcess->addStep('Payment','4000');
    $BasketOrderProcess->addStep('HirePurchase','5000');
    $BasketOrderProcess->addStep('Confirmation','6000');

    # set the last step as responsible for creating the order
    $BasketOrderProcess->setOrderStep('Confirmation');

    return;
}

#========================================================================================
# function     AfterRegisterCheckoutProcess
# state        public
#----------------------------------------------------------------------------------------
# syntax       AfterRegisterCheckoutProcess( $hParams );
#----------------------------------------------------------------------------------------
# description  UI_AfterRegisterCheckoutProcess triggered, to change/remove the basket steps
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>BasketOrderProcess
#                   <li>Servlet
#               </ul> | ref.hash.*
#========================================================================================
sub AfterRegisterCheckoutProcess {
    my ($hParams) = @_;

    my $Servlet = $hParams->{'Servlet'};
    # $Object can be Basket or Order (depends on the settings, if the order is
    # created before external payment is triggered)
    my $Object = $Servlet->object;
    my $BasketOrderProcess = $hParams->{'BasketOrderProcess'};

    # disable HirePurchase Step
    my $ShowHirePurchase = 1;

    $ShowHirePurchase = undef if $ShowHirePurchase and not $Object->container();
    $ShowHirePurchase = undef if $ShowHirePurchase and not $Object->container->get('Payment');
    $ShowHirePurchase = undef if $ShowHirePurchase and not $Object->container->get('Payment')->get('IsHirePurchaseAllowed');
    $BasketOrderProcess->deleteStep('HirePurchase') unless defined $ShowHirePurchase;


    # disable shipping/payment if needed
    my $ShowShipping = 1;
    my $ShowPayment = 1;

    $ShowShipping = undef if $ShowShipping and not $Object->container();
    $ShowShipping = undef if $ShowShipping and not $Object->container->get('NeedShipping');

    $ShowPayment = undef if $ShowPayment and not $Object->container();
    $ShowPayment = undef if $ShowPayment and not $Object->container->get('NeedPayment');
    # show payment alo, if we have maybe a paymentdiscount of 100% like shipping samples etc.
    $ShowPayment = undef if $ShowPayment and ($Object->container->get('PaymentSubTotal') <= 0)
                                         and ($Object->container->get('GrandTotal') <= 0);


    $BasketOrderProcess->deleteStep('Shipping') unless defined $ShowShipping;
    $BasketOrderProcess->deleteStep('Payment') unless defined $ShowPayment;

    # sample for disabling an other step
    #my $ShowOtherStep = 1;
    #$ShowOtherStep = undef unless <condition>;
    #$BasketOrderProcess->deleteStep('OtherStep') unless defined $ShowOtherStep;

    return;
}

#========================================================================================
# function     OnAfterSaveMultiCheckoutBasket
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnAfterSaveMultiCheckoutBasket( $hParams );
#----------------------------------------------------------------------------------------
# description  UI_AfterSaveMultiCheckoutBasket triggered, to test the basket
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Servlet
#               </ul> | ref.hash.*
#========================================================================================
sub OnAfterSaveMultiCheckoutBasket {
    my ($hParams) = @_;

    my $Servlet = $hParams->{'Servlet'};
    my $Basket = $Servlet->object;
    my $ViewAction = 'ViewMultiCheckoutBasket';

    return unless defined $Basket;

    # test basket min order amount
    unless ($Basket->get('IsMinOrderAmountReached')) {

        $Servlet->form->addFormError({
            'Reason'=> 'MinOrderIsNotReached',
            'Name'  => 'Basket',
            'ViewAction' => $ViewAction
        }) ;
    }

    # Add form errors if product is changed and if not available
    my $SkipErrorCheckOnPaid = 0;
    _AddProductLineItemFormErrors($hParams, $ViewAction, $SkipErrorCheckOnPaid);
    $Servlet->form->executeFormError;

    return;
}

#========================================================================================
# function     OnAfterConfirmation
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnAfterConfirmation( $hParams );
#----------------------------------------------------------------------------------------
# description  UI_AfterSaveMultiCheckoutConfirmation  triggered, to test the basket
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Servlet
#               </ul> | ref.hash.*
#========================================================================================
sub OnAfterConfirmation {
    my ($hParams) = @_;
    my $Servlet = $hParams->{'Servlet'};

    # Add form errors if product is changed and if not available
    my $SkipErrorCheckOnPaid = 1;
    _AddProductLineItemFormErrors($hParams, 'ViewMultiCheckoutConfirmation', $SkipErrorCheckOnPaid);
    $Servlet->form->executeFormError;

    return;
}

#========================================================================================
# function     OnSaveMultiCheckoutPayment
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnSaveMultiCheckoutPayment( $hParams );
#----------------------------------------------------------------------------------------
# description  UI_SaveMultiCheckoutPayment  triggered, to test the basket
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Servlet
#               </ul> | ref.hash.*
#========================================================================================
sub OnSaveMultiCheckoutPayment {
    my ($hParams) = @_;

    my $Servlet = $hParams->{'Servlet'};
    my $Basket = $Servlet->object;
    my $Container = $Basket->container->get('Payment');

    # PaymentMethodID from current basket
    my $ContainerPaymentID =  $Container->get('PaymentMethod')->id();
    # PaymentMethodID from selection in checkout payment step
    my $SelectedPaymentID = $Servlet->params('PaymentMethod');

    # Check for LineItemFormErrors only if we go to the confirmation page and not on payment change.
    if ($ContainerPaymentID == $SelectedPaymentID) {

        # Add form errors if product is changed and if not available
        my $SkipErrorCheckOnPaid = 0;
        _AddProductLineItemFormErrors($hParams, 'ViewMultiCheckoutConfirmation', $SkipErrorCheckOnPaid);
        $Servlet->form->executeFormError;
    }

    return;
}

#========================================================================================
# function     OnSaveMultiCheckoutConfirmation
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnSaveMultiCheckoutConfirmation( $hParams );
#----------------------------------------------------------------------------------------
# description  UI_OnSaveMultiCheckoutConfirmation  triggered, to test the basket
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Servlet
#               </ul> | ref.hash.*
#========================================================================================
sub OnSaveMultiCheckoutConfirmation {
    my ($hParams) = @_;

    my $Servlet = $hParams->{'Servlet'};
    my $Basket = $Servlet->object;
    my $ViewAction;
    # in NOW show the CheckoutConfirmationPage again or go back to basket page
    if ($Basket->getSite()->get('IsUnityRegistered')) {
        $ViewAction = 'ViewMultiCheckoutConfirmation';
    } else {
        $ViewAction = 'ViewMultiCheckoutBasket';
    }

    # Add form errors if product is changed and if not available
    my $SkipErrorCheckOnPaid = 1;
    _AddProductLineItemFormErrors($hParams, $ViewAction, $SkipErrorCheckOnPaid);
    $Servlet->form->executeFormError;

    return;
}

#========================================================================================
# function     OnBasket2OrderMultiStepCheckout
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnBasket2OrderMultiStepCheckout( $hParams );
#----------------------------------------------------------------------------------------
# description  UI_Basket2OrderMultiStepCheckout triggered, to test the basket before
#               the order is created
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Servlet
#               </ul> | ref.hash.*
#========================================================================================
sub OnBasket2OrderMultiStepCheckout {
    my ($hParams) = @_;

    # do the same basket test how already done in OnAfterSaveMultiCheckoutBasket
    OnAfterSaveMultiCheckoutBasket($hParams);

    return;
}

#========================================================================================
# function     _AddProductLineItemFormErrors
# state        private
#----------------------------------------------------------------------------------------
# syntax       _AddProductLineItemFormErrors( $aProductLineItems );
#----------------------------------------------------------------------------------------
# description  This function adds form errors if product line item is not available
#               or changed.
#----------------------------------------------------------------------------------------
# input        $aProductLineItems | product line item array as ref
#               <ul>
#                   <li>Servlet
#               </ul> | ref.hash.*
# input        $ViewAction | view action string | string
# input        $SkipErrorCheckOnPaid | boolean (0/1) | integer
#========================================================================================
sub _AddProductLineItemFormErrors {
    my $hParams = shift;
    my ($ViewAction, $SkipErrorCheckOnPaid) = @_;
    my $Servlet = $hParams->{'Servlet'};
    my $Container = $Servlet->object()->container();

    if ($SkipErrorCheckOnPaid) {
        my $LineItemPayment = $Container->get('Payment');
        if ($LineItemPayment->instanceOf('LineItemPaymentExternal') and $LineItemPayment->isPaid()) {
            return;
        }
    }
    my $aProductLineItems = $Container->childrenByClass('LineItemProduct');
    my @DeletedProducts = ();
    my @ProductsMarkedAsChanged;

    foreach my $LineItem ( @{$aProductLineItems} ) {

        my $Product = $LineItem->get('Product');
        if (!defined $Product) {
            push @DeletedProducts, { Name =>$LineItem->get('Name') };
            next
        }

        if ($LineItem->get('IsChanged') == 1) {
            push(@ProductsMarkedAsChanged,{'Name' => $Product->get('NameOrAlias')});
            $Servlet->form->addFormError({
                'Reason' => 'ProductMarkedAsChanged',
                'Name'   => 'ProductAlias',
                'Value'  => $Product->get('NameOrAlias'),
                'ViewAction' => $ViewAction
            });
        }

        if ($Product->get('IsAvailable') == 0) {
            $Servlet->form->addFormError({
                'Reason' => 'ProductNotAvailable',
                'Name'   => 'ProductAlias',
                'Value'  => $Product->get('NameOrAlias'),
                'ViewAction' => $ViewAction
            });
        }
    }

    if (scalar @ProductsMarkedAsChanged) {
        $Servlet->vars('ProductsMarkedAsChangedInChangeAction', 1);
        $Servlet->vars('ProductsMarkedAsChanged', \@ProductsMarkedAsChanged);
    }

    if (scalar @DeletedProducts) {
        # Only throw this form error on the last multi-step checkout page
        if ($ViewAction eq 'ViewMultiCheckoutConfirmation') {
            $Servlet->form->addFormError({
                'Reason'     => 'ProductDeleted',
                'Name'       => 'ProductsDeleted',
                'Value'      => '',
                'ViewAction' => $ViewAction
            });
            $Servlet->vars('DeletedProducts', \@DeletedProducts);
        } else {
            #In ep6 go back to basket page and disable checkout buttons
            $Servlet->vars('DeletedProductsExists', 1);
            $Servlet->vars('ViewAction', $ViewAction);
        }
    }

    return;
}

1;
