#==============================================================================
# package      DE_EPAGES::PayPalPlus::Hooks::Basket
# state        private
#==============================================================================
package DE_EPAGES::PayPalPlus::Hooks::Basket;

use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( LogError );
use DE_EPAGES::Core::API::PerlTools qw ( IsA );
use DE_EPAGES::PayPalPlus::API::Constants qw ( CLASSIC_CHECKOUT PAYPALPLUS_COOKIE );
use DE_EPAGES::PayPalPlus::API::ContextLogger;
use DE_EPAGES::PayPalPlus::API::UserRedirect;
use DE_EPAGES::PayPalPlus::API::Webservice::PatchPayment::Service;

use strict;

#========================================================================================
# function     AfterSaveCustomerData
# state        public
#----------------------------------------------------------------------------------------
# syntax       AfterSaveCustomerData($hParams);
#----------------------------------------------------------------------------------------
# description  call patch payment
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Servlet - servlet - object</li>
#                   <li>Basket - basket - object</li>
#               </ul> | ref.hash.*
#========================================================================================
sub AfterSaveCustomerData {
    my ($hParams) = @_;
    my $MethodName = ( caller(0) )[3];

    my $Servlet = $hParams->{'Servlet'};
    my $Basket = $Servlet->object();
    return unless ( IsA( $Basket, 'DE_EPAGES::Order::API::Object::Basket' ) );

    my $Context = DE_EPAGES::PayPalPlus::API::ContextLogger->new();
    $Context->logContext( $Basket, $MethodName,
        sub {
            eval {
                my $User = $Basket->get('User');
                if( $User->instanceOf('AnonymousUser') ){
                    _PayPalPlus_PatchPayment($hParams);
                } else {
                     # redirect to basket page because we loose the cookie path when the customer
                     # logs in during the classic or mobile checkout.
                    _PayPalPlus_RedirectToBasket($Servlet);
                }
            };
            if (ExistsError()) {
                my $Error = GetError();
                LogError($Error);
            }
        }
    );

    return;
}

#========================================================================================
# function     OnBasket2OrderClassicCheckout
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnBasket2OrderClassicCheckout( $hParams );
#----------------------------------------------------------------------------------------
# description  trigger patch call before basket object becomes an order in classic checkout process
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Servlet - servlet - object</li>
#                   <li>Basket - basket - object</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnBasket2OrderClassicCheckout {
    my ($hParams) = @_;
    my $MethodName = ( caller(0) )[3];

    my $Servlet = $hParams->{'Servlet'};
    return unless defined($Servlet);
    return unless ($Servlet->params('ChangeAction') eq 'TestTermsAndConditions');

    my $Basket = $Servlet->object();
    return unless (defined($Basket) && IsA( $Basket, 'DE_EPAGES::Order::API::Object::Basket' ) );

    my $Context = DE_EPAGES::PayPalPlus::API::ContextLogger->new();
    $Context->logContext( $Basket, $MethodName,
        sub {
            my $StopFlowOnError = 1;
            _PayPalPlus_PatchPayment($hParams, $StopFlowOnError);
        }
    );

    return;
}

#========================================================================================
# function     OnAfterLoginUser
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnAfterLoginUser( $hParams );
#----------------------------------------------------------------------------------------
# description  handle the redirect to the basket page when the customer login
#               during the checkout
#----------------------------------------------------------------------------------------
# input        $hParams | Servlet and OldIntSessionID | refhash
#========================================================================================
sub OnAfterLoginUser {
    my ($hParams) = @_;
    my $MethodName = ( caller(0) )[3];

    my $Servlet = $hParams->{'Servlet'};
    my $Basket = $Servlet->object();
    return unless ( IsA( $Basket, 'DE_EPAGES::Order::API::Object::Basket' ) );

    my $Context = DE_EPAGES::PayPalPlus::API::ContextLogger->new();
    $Context->logContext( $Basket, $MethodName,
        sub {
            eval {
                # redirect to basket page because we loose the cookie path when the customer
                # logs in during the classic or mobile checkout.
                _PayPalPlus_RedirectToBasket($Servlet);
            };
            if (ExistsError()) {
                my $Error = GetError();
                LogError($Error);
            }
        }
    );

    return;
}
#========================================================================================
# function     OnViewBasket
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnViewBasket( $hParams );
#----------------------------------------------------------------------------------------
# description  On classic checkout handle when View Basket
#----------------------------------------------------------------------------------------
# input        $hParams | Servlet and OldIntSessionID | refhash
#========================================================================================
sub OnViewBasket {
    my ($hParams) = @_;
    my $Servlet = $hParams->{'Servlet'};
    my $BasketOrder = $Servlet->object();
    my $LineItemContainer = $BasketOrder->container();
    return unless defined $LineItemContainer;

    my $LineItemPayment = $LineItemContainer->get('Payment');
    return unless defined $LineItemPayment;
    return unless ($LineItemPayment->instanceOf('LineItemPaymentPayPalPlus'));

    # clean paypalplus cookie for the current domain
    $Servlet->cookies(PAYPALPLUS_COOKIE, "=;expires=Thu, 01 Jan 1970 00:00:00 UTC");
    return;
}

#----------------------------------------------------------------------------------------
sub _PayPalPlus_RedirectToBasket {
    my ($Servlet) = @_;

    my $UserRedirect = DE_EPAGES::PayPalPlus::API::UserRedirect->new();
    $UserRedirect->redirectToBasket($Servlet);
    return;
}
#----------------------------------------------------------------------------------------
sub _PayPalPlus_PatchPayment {
    my ($hParams, $StopFlowOnError) = @_;

    my $Servlet = $hParams->{'Servlet'};
    my $Basket = $Servlet->object();
    my $LineItemContainer = $Basket->container();
    my $LineItemPayment = $LineItemContainer->get('Payment');
    if( not IsA( $LineItemPayment, 'DE_EPAGES::PayPalPlus::API::Object::LineItemPaymentPayPalPlus') ) {
        return;
    }

    my $OrderProcess = $Basket->get('Site')->get('OrderProcess');  #classic/multistep
    my $IsMobile = $Servlet->vars('IsMobile');
    if($IsMobile or $OrderProcess == CLASSIC_CHECKOUT) {
        my $PatchPaymentService = DE_EPAGES::PayPalPlus::API::Webservice::PatchPayment::Service->new();
        my $WasSuccessfulRequest = $PatchPaymentService->doRequest($Basket);
        if(not $WasSuccessfulRequest) {
            if ($StopFlowOnError) {
                $IsMobile ? $Servlet->vars('ViewAction', 'MobileSF-View') : $Servlet->vars('ViewAction', 'View');
                $Servlet->form->executeFormError({
                    'Name'   => 'PayPalPlusPayment',
                    'Reason' => 'ConnectionError',
                });
            } else {
                _PayPalPlus_RedirectToBasket($Servlet);
            }
        }

    }
    return;
}

1;