#==============================================================================
# package      DE_EPAGES::PayPalPlus::Hooks::OrderProcess
# state        private
#==============================================================================
package DE_EPAGES::PayPalPlus::Hooks::OrderProcess;

use DE_EPAGES::ExternalPayment::API::Constants qw ( DEFAULT_PAYMENT_STEP_POSITION );
use DE_EPAGES::PayPalPlus::API::Constants qw ( PAYPALPLUS_COOKIE );
use DE_EPAGES::PayPalPlus::API::UserRedirect;

use strict;

#========================================================================================
# function     OnRegisterCheckoutProcess
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnRegisterCheckoutProcess($hParams);
#----------------------------------------------------------------------------------------
# description  register payment step for checkout process
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Servlet - servlet - object</li>
#                   <li>BasketOrderProcess - basket order process - object</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnRegisterCheckoutProcess {
    my ($hParams) = @_;

    my $Servlet = $hParams->{'Servlet'};
    my $BasketOrderProcess = $hParams->{'BasketOrderProcess'};

    my $BasketOrder = $Servlet->object();
    my $LineItemContainer = $BasketOrder->container();
    return unless defined $LineItemContainer;

    my $LineItemPayment = $LineItemContainer->get('Payment');
    return unless defined($LineItemPayment);

    if ($LineItemPayment->instanceOf('LineItemPaymentPayPalPlus')) {
        $BasketOrderProcess->addStep('PayPalPlus', DEFAULT_PAYMENT_STEP_POSITION);
    }

    return;
}



#========================================================================================
# function     AfterRegisterCheckoutProcess
# state        public
#----------------------------------------------------------------------------------------
# syntax       AfterRegisterCheckoutProcess($hParams);
#----------------------------------------------------------------------------------------
# description  set order step; remove payment step if necessary
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Servlet - servlet - object</li>
#                   <li>BasketOrderProcess - basket order process - object</li>
#               </ul> | ref.hash.*
#========================================================================================
sub AfterRegisterCheckoutProcess {
    my ($hParams) = @_;

    my $Servlet = $hParams->{'Servlet'};
    my $BasketOrderProcess = $hParams->{'BasketOrderProcess'};

    my $BasketOrder = $Servlet->object();
    my $LineItemContainer = $BasketOrder->container();
    return unless defined $LineItemContainer;

    my $LineItemPayment = $LineItemContainer->get('Payment');
    return unless defined $LineItemPayment;

    if ($LineItemPayment->instanceOf('LineItemPaymentPayPalPlus')) {
        if ($LineItemPayment->get('PaymentMethod')->get('DoPaymentOnBasket')) {
            $BasketOrderProcess->setOrderStep('PayPalPlus');
        }
    } else {
        $BasketOrderProcess->deleteStep('PayPalPlus');
    }

    return;
}

#========================================================================================
# function     ViewClassicalCheckoutAddress
# state        public
#----------------------------------------------------------------------------------------
# syntax       ViewClassicalCheckoutAddress($hParams);
#----------------------------------------------------------------------------------------
# description  used to restore the paypalplus cookie for 
#               classic and mobile checkout after a domain/ssl switch.
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Servlet - servlet - object</li>
#                   <li>BasketOrderProcess - basket order process - object</li>
#               </ul> | ref.hash.*
#========================================================================================
sub ViewClassicalCheckoutAddress {
    my ($hParams) = @_;

    my $Servlet = $hParams->{'Servlet'};
    my $hFormValues = $Servlet->form()->values();
    my $aChangeActions = $hFormValues->{'ChangeAction'};

    # ensure the hook call action is done only when transitioning from "checkout out" button page
    # because this hook eventually may be called from address page on "next" button if the address form fails,
    # which will always redirect back to "checkout out" button page because parameter "WasPayPalPlusLoadingSuccessful"
    # won't be set to 1.
    return unless ( 'SaveAndDoOrder' ~~ $aChangeActions );

    my $WasPayPalPlusLoadingSuccessful = $Servlet->params('WasPayPalPlusLoadingSuccessful');
    if(not $WasPayPalPlusLoadingSuccessful) {
        my $UserRedirect = DE_EPAGES::PayPalPlus::API::UserRedirect->new();
        my $hAdditionalParams = {'PayPalPlusError' => 1};
        $UserRedirect->redirectToBasket($Servlet, $hAdditionalParams);
        return;
    }

    my $PayPalPlusCookie = $Servlet->form->value(PAYPALPLUS_COOKIE);
    return unless defined $PayPalPlusCookie;

    my %Options = ('HttpOnly' => 0);
    $Servlet->cookies(PAYPALPLUS_COOKIE, $PayPalPlusCookie, \%Options);
}

sub ViewMultiCheckoutConfirmation {
    my ($hParams) = @_;

    my $Servlet = $hParams->{'Servlet'};
    my $WasPayPalPlusLoadingSuccessful = $Servlet->params('WasPayPalPlusLoadingSuccessful');
    if(not $WasPayPalPlusLoadingSuccessful) {
        my $UserRedirect = DE_EPAGES::PayPalPlus::API::UserRedirect->new();
        my $hAdditionalParams = {'PayPalPlusError' => 1};
        $UserRedirect->redirectToPaymentSelectionPage($Servlet, $hAdditionalParams);
    }

    return;
}

1;