use strict;
use Getopt::Long;
use Time::HiRes qw (time);
use DE_EPAGES::Core::API::Error qw( GetError ExistsError );
use DE_EPAGES::Core::API::Log qw( LogError );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( RunOnStore );
use DE_EPAGES::Object::API::Domain qw ( GetUnitySubDomain );
use DE_EPAGES::Object::API::Factory qw ( LoadObject );
use DE_EPAGES::Shop::API::System qw( GetUnityShopIDs );
use DE_EPAGES::WebInterface::API::MessageCenter qw( SynchronizeCache );

#========================================================================================
# function         PatchUnityLogoAndFavicon
# state            public
#----------------------------------------------------------------------------------------
# syntax           PatchUnityLogoAndFavicon();
#----------------------------------------------------------------------------------------
# description      Patches Logo, ShouldHideLogo and FavIcon.
#========================================================================================
sub PatchUnityLogoAndFavicon {
    print "Reading all Unity shops ...\n";
    my $aShopIDs = GetUnityShopIDs();
    foreach my $ShopID (@$aShopIDs) {
        my $Shop = LoadObject($ShopID);

        my $ShopContent;
        eval {
            $ShopContent = $Shop->sendUnityRequest('GET', '/shop');
        };
        if (ExistsError()) {
            print("Patch failed in shop " . $ShopID . " with error \"" . GetError() . "\".\n");
            warn("Patch failed in shop " . $ShopID . " with error \"" . GetError() . "\".\n");
            next;
        }

        my $WebServerScriptNamePart = $Shop->get('WebServerScriptNamePart');
        my $LogoUrl = $ShopContent->{'logoUrl'};
        next unless defined($LogoUrl);

        $LogoUrl = $LogoUrl . '&shop=' . GetUnitySubDomain($WebServerScriptNamePart);
        my $ShouldHideLogo = $ShopContent->{'shouldHideLogo'} | 0; # convert true/false to 1/0
        my $FaviconUrl = $ShopContent->{'faviconUrl'};

        my $hPatchValues = {};
        # set Logo
        # if '/image\?remote=/' in logo path, then logo is saved in epages ui
        if ($Shop->get('Logo') != $LogoUrl and $LogoUrl =~ /image\?remote=/) {
            print "Patch Logo for ". $ShopID . " to " . $LogoUrl . " ...\n";
            $hPatchValues->{'Logo'} = $LogoUrl;
        }

        # set ShouldHideLogo
        if ($Shop->get('ShouldHideLogo') != $ShouldHideLogo) {
            print "Patch ShouldHideLogo for ". $ShopID . " to " . $ShouldHideLogo . " ...\n";
            $hPatchValues->{'ShouldHideLogo'} = $ShouldHideLogo;
        }

        # set FavIcon
        if ($Shop->get('FavIcon') != $FaviconUrl and $FaviconUrl =~ /^\/storage\/images/gi ) {
            print "Patch FavIcon for ". $ShopID . " to " . $FaviconUrl . " ...\n";
            $hPatchValues->{'FavIcon'} = $FaviconUrl;
        }

        $Shop->set($hPatchValues);
    }

    eval { SynchronizeCache(); };
    LogError(GetError()) if ExistsError();

    return;
}

sub Main {

    local $| = 1;

    my $Password;
    my $StoreName;
    my $Help;

    GetOptions(
        'help'        => \$Help,
        'passwd=s'    => \$Password,
        'storename=s' => \$StoreName,
    );

    usage() if $Help;

    unless ( defined $StoreName ) {
        print STDERR "Missing parameter -storename.\n\n";
        usage();
    }
    my $time1 = time;

    RunOnStore(
        Store      => $StoreName,
        DBPassword => $Password,
        Sub        => sub { PatchUnityLogoAndFavicon(); },
    );
    my $time2 = time;
    printf "need %.1f seconds\n", $time2 - $time1;
}

sub usage {
    print <<END_USAGE;
Description:
    Patches Logo, ShouldHideLogo and FavIcon.

Usage:
    perl $0 [options] [flags]

options:
    -passwd     database user password                          ()
    -storename  name of store                                   ()
flags:
    -help       show the command line options

Example:
    perl $0 -storename Store
END_USAGE
    exit 2;
}

RunScript( 'Sub' => \&Main );