#==============================================================================
# package      DE_EPAGES::Shop::UI::Shop
# state        private
#==============================================================================
package DE_EPAGES::Shop::UI::Shop;
use base DE_EPAGES::Presentation::UI::Login;

use strict;
use URI;
use JSON;
use feature                                     qw ( switch );
use HTTP::Status                                qw ( HTTP_SEE_OTHER  );
use DE_EPAGES::Core::API::Error                 qw ( Error ExistsError GetError ResetError );
use DE_EPAGES::Core::API::File                  qw ( DeleteFile );
use DE_EPAGES::Core::API::Log                   qw ( LogDebug LogError );
use DE_EPAGES::Core::API::Image                 qw ( CreateIcon );
use DE_EPAGES::Core::API::PerlTools             qw ( ElementOf trim );
use DE_EPAGES::Core::API::TestParameter         qw ( TestInteger TestString TestUrl TestEMailAddress);
use DE_EPAGES::Core::API::Warning               qw ( Warning );
use DE_EPAGES::Object::API::Constants           qw ( POSITION_START POSITION_STEP );
use DE_EPAGES::Object::API::Domain              qw ( GetUnitySubDomain GetUrlInfo WebServerScriptNamePreview WebServerScriptNameMobileStorefront );
use DE_EPAGES::Object::API::Factory             qw ( Transaction LoadClassByAlias);
use DE_EPAGES::Object::API::Table::Currency     qw ( GetInfoCurrency );
use DE_EPAGES::Object::API::Table::Language     qw ( GetInfoLanguage );
use DE_EPAGES::Object::API::Table::Locale       qw ( GetInfoLocale );
use DE_EPAGES::Presentation::API::User          qw ( GetUsersByEMail );
use DE_EPAGES::Presentation::API::Upload        qw ( Upload );
use DE_EPAGES::Search::API::SearchDefinition    qw ( GetSearchDefinition);
use DE_EPAGES::Search::API::Object::PagerSearch qw ( ReplaceWildcards);
use DE_EPAGES::Shop::API::Shop                  qw ( SaveShopLogo );
use DE_EPAGES::Trigger::API::Trigger            qw ( TriggerHook);
use DE_EPAGES::WebInterface::API::Cookie        qw ( SetCookie );

sub ErrorDefinition {
    return {
        'UnexpectedMobileSFMode' => 'Unexpected MobileSF-Mode was chosen: #Mode.',
    };
}

use constant POSITION_LAST => 999;

#========================================================================================
# function     attributeValues
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hhValues = $Handler->attributeValues( $Servlet, $Object, $Class, $Prefix );
#----------------------------------------------------------------------------------------
# example      $hhValues = $Handler->attributeValues( $Servlet, $Object, $Class, 'Address.' );
#               $Form->executeFormError;
#----------------------------------------------------------------------------------------
# description  Overrides the function of package DE_EPAGES::Presentation::UI::Object
#               to perform some changes before saving the form to the database. For details
#               of the SUPER function, see DE_EPAGES::Presentation::UI::Object package.
#----------------------------------------------------------------------------------------
# input        $Object | object that specifies the context for formatters | object
# input        $Class | class that specifies type and range of the attributes | object
# input        $Object | object that specifies the context for formatters. For existing
#                           objects this is the object that is to be saved; for new
#                           objects this may be a direct or indirect parent object.
#                        | object
# input        $Prefix | prefix for input field names (default '') | string
#-----------------------------------------------------------------------------------------
# return       $hhValues | checked attribute values
#                   <ul>
#                       <li>Values - non-localized attribute values execpt files
#                                   { $AttributeName => $Value }</li>
#                       <li>Files - non-localized attribute values of type File
#                                   { $AttributeName => $FileName }
#                                   if the file name is undef, then the file is about to be deleted</li>
#                       <li>LocalizedValues - localized attribute values execpt files
#                                   { $LanguageID => { $AttributeName => {'Value'=>$Value, 'Index'=>$Index } }</li>
#                       <li>LocalizedFiles - localized attribute values of type File
#                                   { $LanguageID => { $AttributeName => $FileName } }
#                                   if the file name is undef, then the file is about to be deleted</li>
#                   </ul>
#                   | ref.hash.hash
#========================================================================================
sub attributeValues{
      my $self = shift;
      my ( $Servlet, $Object, $Class, $Prefix ) = @_;
      my $hhValues = $self->SUPER::attributeValues( $Servlet, $Object, $Class, $Prefix );

      # If the GrantServiceAccess is not allowed  the input field of 'GrantServiceAccessUntil'
      # is disabled and will not be sent with form. This code fix this.
      # EPG-4091
      if(defined $Servlet->form->value('GrantServiceAccessUntil_RadioSwitch')
            && $Servlet->form->value('GrantServiceAccessUntil_RadioSwitch') == 0) {
       $hhValues->{'Values'}->{'GrantServiceAccessUntil'} = undef;
      }
      return $hhValues;
}


#==============================================================================
# Event handlers
#==============================================================================

sub UnityViewLanguages {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;

    #loop ShopLanguages and get default and active language (locales)
    my $ahShopLanguages = [];
    my $hShopDefaultLanguage = {};
    my $ShopLanguageID = $Shop->get('LanguageID');
    my $aSiteLocales   = $Shop->get('SiteLocales');
    my $aSiteLocaleIDs =  [map {$_->get('LocaleID')} @$aSiteLocales] ;

    # test if we have the default language and an active locale or not
    foreach my $Language (@{$Shop->folder('Languages')->children()}){
        # is this the default language? then fill the hash
        # because we displayy the default language as 1st entry in the list
        if ($ShopLanguageID eq $Language->get('LanguageID')) {
            $hShopDefaultLanguage = $Language->tleHash;
            next;
        }

        # test if the current language with the according locale is active
        # and add the language to the list
        my $IsActive = 0;
        my $hValues = $Language->get(['Locales', 'LanguageID', 'Native', 'English', 'Code3']);
        foreach my $hLocale (@{$hValues->{'Locales'}}) {
            $IsActive = 1 if ElementOf($hLocale->{'LocaleID'}, @$aSiteLocaleIDs);
        }

        push @$ahShopLanguages , {'LanguageID'  => $hValues->{'LanguageID'},
                                  'Native'      => $hValues->{'Native'},
                                  'English'     => $hValues->{'English'},
                                  'Code3'       => $hValues->{'Code3'},
                                  'IsNotActive' => $IsActive ? 0:1,
                                  };
    }

    # write default language and other languages to the servlet
    $Servlet->vars('ShopDefaultLanguage',$hShopDefaultLanguage);
    $Servlet->vars('ShopLanguages',$ahShopLanguages);

}

sub UnityDeleteLanguages {
    my $self = shift;

    $self->DeleteLanguages(@_);

    # we need to inform UNITY-SF about the deletion of the language

}

sub UnityLanguagesSetVisible {
    my $self = shift;
    my ($Servlet) = @_;

    my @Languages = $Servlet->form->value('LanguageID');
    return if !scalar @Languages;
    my $Shop = $Servlet->object;

    # set language visible, that means we  get the language id from template and add
    # the matching Locale: see comment in localeForLanguageByShopLocaleIDAndLanguage()
    foreach my $LanguageID (@Languages){
        # load the language by id
        my $aShopLanguages = LoadClassByAlias('ShopLanguage')->getObjects('ByShopIDLanguageID', [$Shop->id, $LanguageID]);
        # get locale and add to shop
        my $LocaleIDForNewShop = $Shop->localeForLanguageByShopLocaleIDAndLanguage($aShopLanguages->[0]);
        $Shop->addLocale($LocaleIDForNewShop) unless $Shop->existsLocale($LocaleIDForNewShop);
        $Shop->sendUnityRequest('PATCH', '/shop/locale/' . $LocaleIDForNewShop, undef, { 'isVisible' => JSON::true });
    }

    return;
}

sub UnityLanguagesSetInvisible {
    my $self = shift;
    my ($Servlet) = @_;

    # set language invisible, that means we delete the locale for the language
    # (UNITY-SF can also get the locales by REST)

    my @Languages = $Servlet->form->value('LanguageID');
    return if !scalar @Languages;
    my $Shop = $Servlet->object;

    # get the locale for each laguage and delete the locale
    foreach my $LanguageID (@Languages){
        # remove locales for language
        my @Locales = grep { $_->get('Language')->{'LanguageID'} == $LanguageID} @{$Shop->get('SiteLocales')};
        foreach my $Locale (@Locales) {
            my $LocaleID = $Locale->get('LocaleID');
            $Shop->removeLocale($LocaleID);
            $Shop->sendUnityRequest('PATCH', '/shop/locale/' . $LocaleID, undef, { 'isVisible' => JSON::false });
        }
    }

    return;
}

sub UnitySaveLanguage {
    my $self = shift;
    my ($Servlet) = @_;

    # we get the language id from template and add the language to Shop.Languages
    # then we add the matching Locale: see comment in _activateLocaleForLanguage()
    my $Shop = $Servlet->object;

    # add language to shop
    if( $Servlet->params('NewLanguageID') ) {
        my $LanguageID = $Servlet->params('NewLanguageID');
        unless( $Shop->existsLanguage($LanguageID) ) {
            my $NewLanguage;
            eval {
                Transaction( sub {
                    $NewLanguage = $Shop->addLanguage($LanguageID);
                } );
            };
            if (ExistsError()) {
                my $Error = GetError();
                if ($Error->code  eq 'FeatureLimitExceeded'){
                    $Servlet->vars('FeatureMaxValueReached', $Error->vars);
                    Warning($Error);
                } else {
                    Error($Error);
                }
            } else {
                $Servlet->vars('AddedLanguage',$NewLanguage->tleHash);
                my $LocaleIDForNewShop = $Shop->localeForLanguageByShopLocaleIDAndLanguage($NewLanguage);

                # now add the matching locale to the shop
                unless ($Shop->existsLocale($LocaleIDForNewShop)){
                    $Shop->addLocale($LocaleIDForNewShop);

                    # add the country from the locale for the language to the country list here.....
                    # test if we have already the according country in the shop
                    my $CountryIDFromLocale = GetInfoLocale($LocaleIDForNewShop)->{'CountryID'};
                    unless ($Shop->existsCountry($CountryIDFromLocale)) {
                        $Shop->addCountry($CountryIDFromLocale,POSITION_LAST,0);
                        $Shop->renumberCountries();
                        $Servlet->vars('AddedLocaleCountry', $Shop->siteLocale($LocaleIDForNewShop)->get('Country'));
                    }
                }
            }
        }

    }

    return;

}

sub DeleteLanguages {
    my $self = shift;
    my ($Servlet) = @_;

    my @Languages = $Servlet->form->value('LanguageID');
    return if !scalar @Languages;
    my $Shop = $Servlet->object;

    # prevent deleting of all languages
    return $Servlet->vars('DELETE_LAST',1) if scalar(@Languages) == scalar(@{$Shop->languages});

    # prevent deleting of all backoffice languages
    my @BOLanguagesIDs = map { $_->{'LanguageID'} } grep {$_->{'IsForBackoffice'} == 1 } @{$Shop->get('Languages')};
    my $DeleteBOCount = 0;

    foreach my $LanguageID (@Languages){
       $DeleteBOCount ++ if ElementOf($LanguageID,@BOLanguagesIDs);
    }
    return $Servlet->vars('DELETE_LAST_BO',1) if $DeleteBOCount == scalar(@BOLanguagesIDs);

    # delete only if accepted if confirmation is used
    if ($Servlet->user->get('DeleteConfirmation') and not $Servlet->params('ACCEPT')) {
        # set delete mark to template
        my @DeleteObjects;
        foreach my $Item (@Languages) {
            push @DeleteObjects, GetInfoLanguage($Item);
        }
        $Servlet->vars('DeleteObjects',\@DeleteObjects);
        $Servlet->vars('DELETE',1);
        return;
    } else {
        # else reset delete
        $Servlet->vars('DELETE',0);
    }

    $Shop->removeLanguage($_) foreach @Languages;
    $Shop->updatePageCache;
    return;
}

sub SaveLanguage {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;

    # add new locales
    my @LocaleIDs = $Servlet->form->value('LocaleID');
    foreach my $LocaleID (@LocaleIDs) {
        # skip locale if the language is not enabled or language pack is not installed
        next unless $Shop->existsSiteLocale( $LocaleID );

        $Shop->addLocale($LocaleID) unless $Shop->existsLocale($LocaleID);
    }

    # update default locale for available languages
    if ($Servlet->params('Default')) {
        my $LocaleID = $Servlet->params('Default');

        # check that the new default locale is a valid storefront locale
        if( $Shop->existsLocale( $LocaleID ) ) {
            $Shop->set({'LocaleID' => $LocaleID});
        }
        else {
            $Servlet->form->executeFormError({
                'Reason'=> 'LOCALE_NOT_ACTIVE',
                'Form'  => 'Form',
                'Name'  => 'LocaleID',
                'Value' => $LocaleID,
            });
        }
    }

    # test if we have a backoffice language and set the site locales

    # do not remove default locale
    my %NewLocaleIDs = map { $_ => 1 } @LocaleIDs;
    if( !exists $NewLocaleIDs{$Shop->get('LocaleID')} ) {
        $Servlet->form->executeFormError({
            'Reason'=> 'CANNOT_DELETE_DEFAULT',
            'Form'  => 'Form',
            'Name'  => 'LocaleID',
        });
    }

    # test Position for languages (unless they are empty)
    my $aShopLanguages = $Shop->child('Languages')->children;
    my @LanguagePositions = $Servlet->form->value('LanguagePosition');
    my $Index = 0;
    if( scalar @$aShopLanguages == scalar @LanguagePositions ) { # EPG-13496
        foreach my $Position (@LanguagePositions) {
            $Position = $Servlet->form->field($Shop, $Position, {
                'Name' => 'LanguagePosition',
                'Type' => 'reg_integer',
                'LoopName' => 'LoopName',
                'Index' => $Index,
                'Mandatory' => 0,
            });

            $Index++;
        }
    }

    # test Postion for locales (unless they are empty)
    my @Locales = $Servlet->form->value('Locales');
    my @LocalePositions = $Servlet->form->value('LocalePosition');
    $Index = 0;
    foreach my $Position (@LocalePositions) {
        $Position = $Servlet->form->field($Shop, $Position, {
            'Name' => 'LocalePosition',
            'Type' => 'reg_integer',
            'LoopName' => 'LoopName',
            'Index' => $Index,
            'Mandatory' => 0,
        });
        $Index ++;
    }

    $Servlet->form->executeFormError;

    # remove locales
    foreach my $LocaleID ( map { $_->get('LocaleID') } @{$Shop->get('SiteLocales')} ) {
        $Shop->removeLocale($LocaleID) unless exists $NewLocaleIDs{$LocaleID};
    }

    #set position for locales
    $Index = 0;
    foreach my $LocaleID (@Locales) {
        if ($Shop->existsLocale($LocaleID)) {
            my $Position = $LocalePositions[$Index];
            $Shop->updateSiteLocale($LocaleID, $Position) if defined $Position;
        }
        $Index++;
    }

    # set position for languages
    if( scalar @$aShopLanguages == scalar @LanguagePositions ) { # EPG-13496
        $Index = 0;
        foreach my $Language (@$aShopLanguages) {
            my $Position = $LanguagePositions[$Index];
            $Language->set({'Position' => $Position}) if defined $Position;
            $Index++;
        }
    }

    # add language to shop
    if( $Servlet->params('NewLanguageID') ) {
        my $LanguageID = $Servlet->params('NewLanguageID');
        unless( $Shop->existsLanguage($LanguageID) ) {
            eval {
                Transaction( sub {
                    $Shop->addLanguage($LanguageID);
                } );

                # if only one language is allowed, the user cannot select the locales
                # therefore the new locales will be set automatically
                @LocaleIDs = map { $_->get('LocaleID') } @{$Shop->get('SiteLocales')};
            };
            if (ExistsError()) {
                my $Error = GetError();
                if ($Error->code  eq 'FeatureLimitExceeded'){
                    $Servlet->vars('FeatureMaxValueReached', $Error->vars);
                    Warning($Error);
                } else {
                    Error($Error);
                }
            }
        }
    }

    #renumber positions
    my $Position = POSITION_START;
    $aShopLanguages = $Shop->child('Languages')->children;
    foreach my $Language (@$aShopLanguages) {
        $Language->set({'Position' => $Position});
        $Position += POSITION_STEP;
    }
    my $aLocales = $Shop->get('SiteLocales');
    $Position = POSITION_START;
    foreach my $hLocale (@$aLocales) {
        $Shop->updateSiteLocale($hLocale->get('LocaleID'), $Position);
        $Position += POSITION_STEP;
    }

    $Shop->updatePageCache;

    return;
}

sub AddCurrency {
    my $self = shift;
    my ($Servlet) = @_;
    return if ! $Servlet->params('NewCurrencyID');

    my $Shop = $Servlet->object;
    my $CurrencyID = $Servlet->params('NewCurrencyID');

    eval {
        $Shop->addCurrency($CurrencyID) if ! $Shop->existsCurrency($CurrencyID);
    };
    if (ExistsError()) {
        my $Error = GetError();
        if ($Error->code  eq 'FeatureLimitExceeded'){
            $Servlet->vars('FeatureMaxValueReached', $Error->vars);
            Warning($Error);
        } else {
            Error($Error);
        }
    }
    return;
}

sub DeleteCurrencies {
    my $self = shift;
    my ($Servlet) = @_;
    return if ! $Servlet->params('CurrencyID');
    my @Currencies = $Servlet->form->value('CurrencyID');
    my $Shop = $Servlet->object;

    # prevent deleting of all currencies
    return $Servlet->vars('DELETE_LAST',1) if scalar(@Currencies) == scalar(@{$Shop->currencies});

    # delete only if accepted if confirmation is used
    if ($Servlet->user->get('DeleteConfirmation') and not $Servlet->params('ACCEPT')) {
        # set delete mark to template
        my @DeleteObjects;
        foreach my $Item (@Currencies) {
            push @DeleteObjects, GetInfoCurrency($Item);
        }
        $Servlet->vars('DeleteObjects',\@DeleteObjects);
        $Servlet->vars('DELETE',1);
        return;
    } else {
        # else reset delete
        $Servlet->vars('DELETE',0);
    }
    $Shop->removeCurrency($_) foreach @Currencies;
    $Servlet->vars('DELETE',0);

    # set new default currency and activate this
    $Shop->set({'CurrencyID' => ${$Shop->currencies}[0]}) if scalar @{$Shop->currencies};
    $Shop->activateCurrency(${$Shop->currencies}[0]) if scalar @{$Shop->currencies};
    $Shop->updatePageCache;

    return;
}

sub SaveCurrency {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object;

    if( $Servlet->params('NewCurrencyID') ) {
        my $CurrencyID = $Servlet->params('NewCurrencyID');

        # if only one currency is permitted, remove the current currency before adding a new one
        my $FeatureMaxValue = $Shop->featureMaxValue( 'Currencies' );
        my $aCurrencies = $Shop->currencies;

        if( 1 == @$aCurrencies && 1 == $FeatureMaxValue && $aCurrencies->[0] ne $CurrencyID ) {
            Transaction( sub {
                $Shop->removeCurrency( $aCurrencies->[0] );
                $Shop->addCurrency($CurrencyID);
                $Shop->activateCurrency($CurrencyID);
                $Shop->set({'CurrencyID' => $CurrencyID});
            } );
            return; # don't save the visible/default state
        }
        else {
            $self->AddCurrency( $Servlet );
        }
    }

    my @ActiveIDs = $Servlet->form->value('Active');
    my $DefaultCurrency = $Servlet->params('Default');
    if (defined $DefaultCurrency) {
        # test if default is visible
        return $Servlet->vars('INVISIBLE_DEFAULT',$DefaultCurrency) if ! ElementOf($DefaultCurrency,@ActiveIDs);
        $Shop->set({'CurrencyID' => $DefaultCurrency});
    }

    foreach my $CurrencyID (@{$Shop->currencies}) {
       if (ElementOf($CurrencyID,@ActiveIDs)){
           $Shop->activateCurrency($CurrencyID) ;
       }else {
           $Shop->deactivateCurrency($CurrencyID) ;
       }
    }

    my $hForm = $Servlet->form->form($Shop, 'ListedCurrencies');
    foreach my $hItem( @{$hForm->{'ListCurrencies'}} ) {
        $Shop->updateCurrency($hItem->{'Currency'}, $hItem->{'ListPosition'});
    }

    #renumber positions
    my $Position = POSITION_START;
    foreach my $CurrencyID (map {$_->{'CurrencyID'}} @{$Shop->get('Currencies')}) {
        $Shop->updateCurrency($CurrencyID, $Position);
        $Position += POSITION_STEP;
    }

    $Shop->updatePageCache;
    return;
}

sub DeleteLocales {
    my $self = shift;
    my ($Servlet) = @_;
    return if ! $Servlet->params('ListObjectID');
    my @Locales = $Servlet->form->value('ListObjectID');
    my $Shop = $Servlet->object;

    # prevent deleting of all currencies
    return $Servlet->vars('DELETE_LAST',1) if scalar @Locales == scalar @{$Shop->get('SiteLocales')};
    $self->DeleteListedObjects($Servlet);
    if (not $Shop->folder('Locales')->existsChild($Shop->get('LocaleID'))) {
        my $aLocales = $Shop->get('SiteLocales');
        $Shop->set({'LocaleID'=>$aLocales->[0]->alias});
    }
    return;
}

sub SetAdminUsersInActive {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;
    $self->TestIsSuperMerchant($Servlet);

    my @ListObjectIDs = $Servlet->form->value('ListObjectID');

    foreach my $ListObject (@ListObjectIDs){
        my $User = $Shop->loadObject($ListObject, 'User');
        $Servlet->testPermission($User, 'Save');
        $User->set({'IsActivated' => 0});
    }

    return;
}

sub SetAdminUsersActive {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;
    $self->TestIsSuperMerchant($Servlet);

    my @ListObjectIDs = $Servlet->form->value('ListObjectID');

    foreach my $ListObject (@ListObjectIDs){
        my $User = $Shop->loadObject($ListObject, 'User');
        $Servlet->testPermission($User, 'Save');
        $User->set({'IsActivated' => 1});
    }

    return;
}

sub TestUserManagerFeature {
    my $self = shift;
    my ($Shop) = @_;

    if( 0 == $Shop->featureMaxValue('UserManager') ) {
        Error( 'FeatureLimitExceeded', {
            Feature         => 'UserManager',
            CurrentValue    => $Shop->featureCurrentValue('UserManager'),
            MaxValue        => 0,
            NewValue        => 1,
            Shop            => $Shop->alias,
            Name => 'UserManager',
        });
    }
    return;
}

sub TestIsSuperMerchant {
    my $self = shift;
    my ($Servlet) = @_;

    my $CurrentUser = $Servlet->user;
    if(not $CurrentUser->get('IsSuperMerchant') ){
        $Servlet->form->executeFormError({
            'Reason'=> 'ACTION_ILLIGAL',
            'Form'  => 'NewAdminUser',
            'Name'  => 'Action'
        });
    }
    return;
}

sub testUserData {
    my $self = shift;
    my ($Servlet,$Email, $Name) = @_;

    my $Shop = $Servlet->object;

    trim($Email);
    $Servlet->form->addFormError({
        'Reason'=> 'PARAMETER_MISSING',
        'Form'  => 'Form',
        'Name'  => 'Email',
    }) unless $Email;

    trim($Name);
    $Servlet->form->addFormError({
        'Reason'=> 'PARAMETER_MISSING',
        'Form'  => 'Form',
        'Name'  => 'Name',
    }) unless $Name;

    eval {
        TestEMailAddress( 'EMail', $Email );
    };
    if (ExistsError) {
        $Servlet->form->executeFormError({
            'Reason'=> 'FORMAT_NOT_EMAILADDRESS',
            'Name'  => 'EMail',
            'Form'  => 'Form',
        });
    }

    $Servlet->form->executeFormError;

    # check if the email is now in use
    $Servlet->form->executeFormError({
        'Reason'=> 'EMAIL_EXISTS',
        'Form'  => 'Form',
        'Name'  => 'Email',
    }) if scalar @{GetUsersByEMail($Email, $Shop)};

    return $Email,$Name;
}

sub NewAdminUser {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;

    $self->TestIsSuperMerchant($Servlet);
    $self->TestUserManagerFeature($Servlet->object);
    my ($Email,$Name) = $self->testUserData($Servlet, $Servlet->params('Email'),$Servlet->params('Name'));

    my %UserParams = (
        'EMail' => $Email,
        'Name' => $Name,
        'IsActivated' => 0,
        'ShowFunctionHelp' => 0,
        'ShowSidebar' => 1,
    );

    Transaction( sub {
        my $UserFolder = $Shop->child('Users');
        my $User = $UserFolder->insertChild( 'User', \%UserParams);
        my $MerchantGroup = $Shop->child('Groups')->child('Merchants');
        $MerchantGroup->addMember($User);
    });

    $Servlet->vars('USER_EMPTY_PASSWD',1);

    return;
}

sub NewUser {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;
    my $hValues = $Servlet->form->form($Shop, 'NewUser');

    my ($Email,$Name) = $self->testUserData($Servlet,$hValues->{'EMail'},$hValues->{'Name'});

    # create new User/AdminUser with the status inactive
    my %UserParams = (
        'EMail' => $Email,
        'Name' => $Name,
        'IsActivated' => 0,
    );

    Transaction( sub {
        my $UserFolder = $Shop->child('Users');
        my $User = $UserFolder->insertChild( 'User', \%UserParams);
        if ($hValues->{'GroupID'}){
            my $Group = $Shop->loadObject($hValues->{'GroupID'}, 'Group');
            die "GroupID $hValues->{'GroupID'} does not belong to shop ".$Shop->alias unless $Group->getSite->id == $Shop->id;
            $Group->addMember($User);
        }
    });

    $Servlet->vars('USER_EMPTY_PASSWD',1);

    return;
}


sub SavePageCacheSettings {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object;

    my $hForm = $Servlet->form->form($Shop, 'PageCache');
    my $IsPageCacheUsed = $hForm->{'IsPageCacheUsed'};
    $Shop->clearPageCache() if $IsPageCacheUsed and not $Shop->get('IsPageCacheUsed');
    $Shop->set({ 'IsPageCacheUsed'=> $IsPageCacheUsed ? 1 : 0});

    my %Shop = ();
    foreach my $Key (grep { defined $hForm->{$_} } keys %$hForm){
        if ($Key =~ /^CachedPages\.(.+)/) {
            $Shop{'CachedPages'}{$1} = $hForm->{$Key};
        }
    }
    $Shop->set(\%Shop) if scalar keys %Shop;
    return;
}

sub ClearPageCache {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object;

    $Shop->clearPageCache();
    $Servlet->vars('PAGECACHE_CLEARED', 1);
    return;
}

sub Publish {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object;

    # activate the page cache (don't change the current settings)
    $Shop->set({ 'IsPageCacheUsed' => 1 }) unless $Shop->get('IsPageCacheUsed');

    $Shop->clearPageCache();
    $Servlet->vars('PAGECACHE_CLEARED', 1);

    # start publishing
    $Servlet->vars('SHOP_PUBLISHED', 1);
    return;
}

sub SaveFiles {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object;

    my $Form = $Servlet->form->form($Shop, 'SaveFiles');

    # save new image
    if ($Form->{'NewFile'}){
        # is URL saved and contains //hostname
        my (undef, $hResponseContent) = Upload($Shop, $Form, {'FormName' => 'SaveFiles', 'InputName' => 'NewFile', 'Index' => 0, 'Directory' => '/MediaGallery/'});
        $Servlet->vars('ResponseContent', $hResponseContent);
    }

    # delete File
    if ($Servlet->params('Delete')){
        my $aFiles = $Form->{'Files'};
        return unless scalar @{$aFiles};
        if ($Servlet->user->get('DeleteConfirmation') and not $Servlet->params('ACCEPT')) {
            # set delete mark to template
            my @DeleteFiles;
            foreach my $File (@{$aFiles}) {
                my %Vars;
                $Vars{'DeleteFile'} = $File->{'DeleteFile'};
                push @DeleteFiles,\%Vars;
            }
            $Servlet->vars('DeleteFiles',\@DeleteFiles);
            $Servlet->vars('DELETE',1);
            return;
        } else {
            # else reset delete
            $Servlet->vars('DELETE',0);
        }
        # delete images
        foreach my $File (@$aFiles) {
            if ($File->{'DeleteFile'}) {
                my $hFileInfo;
                # is URL saved and contains //hostname
                if ($File->{'DeleteFile'} =~ /\.url/) {
                    ($hFileInfo) = grep { $_->{'Extension'} eq '.url' and
                                             $_->{'URLFileName'} eq $File->{'DeleteFile'}
                                           } @{$Shop->get('MediaFiles')};
                } else {
                    ($hFileInfo) = grep { $_->{'FileName'} eq $File->{'DeleteFile'} } @{$Shop->get('MediaFiles')};
                }
                if ( defined $hFileInfo ) {
                    DeleteFile($hFileInfo->{'Name'});
                }
            }
        }
    }

    return {};
}

sub Save {
    my $self = shift;
    my ($Servlet) = @_;
    $self->SaveLogo($Servlet);
    $self->SUPER::Save($Servlet);
    $self->SaveMobileSFMode($Servlet);

    # invaidate cache to prevent gross/net template errors
    $Servlet->object()->invalidatePageCache;

    return;
}

sub SaveMobileSFMode {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object();
    my $MobileSFMode = $Servlet->form->value('MobileSFModeShop');

    if(defined $MobileSFMode){
        if(!$MobileSFMode) {
            $Shop->set({'UseMobileSF' => 0});
        }
        elsif($MobileSFMode == 1) {
            $Shop->set({'MobileSFModeShop' => 0, 'UseMobileSF' => 1});
        }
        elsif($MobileSFMode == 2) {
            my $hParams = {
                'Shop' => $Shop,
                'DefaultSFMode' => 0,
            };
            TriggerHook('UI_SaveMobileSFMode', $hParams);
            if($hParams->{'DefaultSFMode'}){
                $Shop->set({'MobileSFModeShop' => 1, 'UseMobileSF' => 1});
            } else {
                $Servlet->form->addFormError({
                    'Reason' => 'MobileSFModeNoBasketFeature',
                    'Name'   => 'MobileSFModeShop',
                });
            }
        } else {
            Error('UnexpectedMobileSFMode', {'Mode' => $MobileSFMode});
        }
    }

    $Servlet->form->executeFormError;
    return;
}


sub SaveLogo {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object();
    my $Form = $Servlet->form();

    my $Logo;
    eval {
        $Logo = SaveShopLogo($Shop, $Form);
    };
    if (ExistsError()) {
        my $Error = GetError();
        if ($Error->code eq 'UNITY_API_CALL_ERROR') {
            LogError($Error);
            $Servlet->form->executeFormError({
                'Reason'=> 'UNITY_API_CALL_ERROR',
                'Name'  => 'Logo',
                'GUID'  => $Error->guid
            });
        } else {
            Error($Error);
        }
    }

    if (defined $Logo && $Shop->get('IsUnityRegistered') && $Form->value('Logo')) {
        $Servlet->vars('Logo', $Logo);
        # set JSON output, too
        $Servlet->vars('ResponseContent', {
            'FileName' => $Shop->get('WebPath') . '/' . $Logo,
            'Logo' => $Logo
        });
    }

    $Servlet->form->executeFormError;
    return;
}

sub SaveUserGroup {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object;

    NewUserGroup($self,$Servlet) if $Servlet->params('NewAlias');

    my $Form   = $Servlet->form->form($Shop, 'SaveUserGroups');
    foreach my $LanguageEntry (@{$Form->{'Languages'}}) {
        my %Info;
        $Info{$_} = $LanguageEntry->{$_} foreach keys %$LanguageEntry;
        delete $Info{'LanguageID'};
        delete $Info{'UserGroupID'};
        $Shop->loadObject($LanguageEntry->{'UserGroupID'}, 'Group')->set(\%Info, $LanguageEntry->{'LanguageID'});
    }

    return;
}


sub NewUserGroup {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop   = $Servlet->object;
    my $Groups = $Shop->child('Groups');
    my $Form = $Servlet->form->form($Shop, 'NewUserGroup');

    my $UserGroup = undef;
    if (! $Groups->existsChild( $Form->{'NewAlias'} ) ){
        $UserGroup = $Groups->insertChild('Group',{'Alias'=>$Form->{'NewAlias'}});
    } else{
        $Servlet->vars('USERGROUP_EXISTS',1);
            $Servlet->form->executeFormError({
            'Reason'=> 'USERGROUP_EXISTS',
            'Form'  => 'NewUserGroup',
            'Name'  => 'NewAlias',
            'ItemNo'=> 0,
            'Value' => $Form->{'NewAlias'},
            }) ;
        return;
    }

    foreach my $LanguageEntry (@{$Form->{'Languages'}}) {
        my %Info;
        $Info{$_} = $LanguageEntry->{'New'.$_} foreach map { substr($_, 3) } keys %$LanguageEntry;
        delete $Info{'LanguageID'};
        $UserGroup->set(\%Info, $LanguageEntry->{'NewLanguageID'});
    }
    return;
}



sub NewRegion {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;

    my $NewAlias = $Servlet->params('NewAlias');

    if ( $NewAlias =~ /\S/ ) {
        # test if region with the given name exists
        my $Folder = $Shop->folder('Regions');
        if ($NewAlias && $Folder->existsChild($NewAlias)){
            $Servlet->vars('OBJECT_EXISTS',1);
            $Servlet->form->executeFormError({
                'Reason'=> 'OBJECT_EXISTS',
                'Name'  => 'NewAlias',
                'Value' => $NewAlias,
            }) ;
        }

        eval {
            $Shop->addRegion($NewAlias);
        };
        if (ExistsError()) {
            my $Error = GetError();
            Error($Error) unless($Error->code eq 'FeatureLimitExceeded');
            ResetError();

            $Servlet->vars('FeatureMaxValueReached', $Error->vars);
        }

    }
    return;
}

sub UnityNewRegion {
    my $self = shift;
    my ($Servlet) = @_;

    my $NewAlias = $Servlet->params('NewAlias');
    $Servlet->form->addFormError({
        'Reason'=> 'AliasMissing',
        'Form'  => 'Form',
        'Name'  => 'Alias',
    }) unless $NewAlias;

    my @CountryIDs = $Servlet->form->value('ListObjectID');
    $Servlet->form->addFormError({
        'Reason'=> 'NoCountrySelected',
        'Form'  => 'Form',
        'Name'  => 'ListObjectID'
    }) unless scalar @CountryIDs;

    $Servlet->form->executeFormError();

    $self->NewRegion($Servlet);

    my $Shop = $Servlet->object;
    my $Region = $Shop->folder('Regions')->child($NewAlias);

    $Region->saveCountries(\@CountryIDs);

    $self->SUPER::Save($Servlet);

    return;
}

sub DeleteRegions {
    my $self = shift;
    my ($Servlet) = @_;

    $self->SUPER::DeleteListedObjects($Servlet);
    return;
}

sub ViewBusinessHours {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;
    my $Calendar = $Shop->businessHours;
    $Servlet->vars('ObjectID', $Calendar->id);
    $self->SUPER::View($Servlet);
    return;
}


sub GenerateFavIcon {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object();
    my $PublicPath = $Shop->get('PublicPath');
    my $Logo = $Shop->get('Logo');
    if (defined $Logo) {
        CreateIcon($PublicPath.'/'.$Logo, $PublicPath.'/'.'favicon.ico');
        $Shop->set({'FavIcon'=>'favicon.ico'});
    }
    return;
}

sub ViewUserManagerUsers {
    my $self = shift;
    my (@Params) = @_;
    return $self->SearchUsers(@Params);
}


sub SearchUsers {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object;
    my $SearchForm = $Servlet->form->form($Shop, 'SearchUsers');

    delete $SearchForm->{$_} foreach grep { not defined $SearchForm->{$_} } keys %$SearchForm;
    my $SearchName = $SearchForm->{'Search'} || 'Users';
    my $SearchDefinition = GetSearchDefinition(LoadClassByAlias('User'), $SearchName);

    my $OrderDesc = $Servlet->params('OrderDesc') // 1;
    $Servlet->vars('OrderDesc',$OrderDesc);

    my %Options = (
        'Servlet' => $Servlet,
        'Object'  => $Shop,
        'SearchDefinition' => $SearchDefinition,
        'Parameters' => {
            'Shop.ID'    => $Shop->id,
            'LanguageID' => $Servlet->languageID,
            'ExcludeGroupIDs' => [ map { $_->id } @{$Shop->get('NonStorefrontUserGroups')} ],
            'UserClass.ID' => LoadClassByAlias('User')->id,
        },
        'OrderBy'    => 'CreationDate',
        'OrderDesc'  => $OrderDesc,
        'UriParameters'  => {
            'Search' => $SearchName,
        },
    );
    delete $SearchForm->{'Search'};
    foreach my $Parameter (keys %$SearchForm) {
        if ($Parameter =~ /String/) {

            # replace * with %
            $Options{'Parameters'}{$Parameter} = ReplaceWildcards($SearchForm->{$Parameter});
        } elsif ($Parameter eq 'CustomerGroupID') {
            $Options{'Parameters'}{'CustomerGroup.ID'} = $SearchForm->{$Parameter};
        }else{
            $Options{'Parameters'}{$Parameter} = $SearchForm->{$Parameter};
        }
        $Options{'UriParameters'}{$Parameter} = $SearchForm->{$Parameter};
    }

    $Servlet->vars('IS_SEARCH',1) if scalar keys(%$SearchForm);

    LogDebug('SearchForm', $SearchForm);
    LogDebug('Parameters', $Options{'Parameters'});

    my $Search = DE_EPAGES::Search::API::Object::PagerSearch->new(\%Options);
    return $Search->page;
}

sub DeleteListedUsers {
    my $self = shift;
    my ( $Servlet ) = @_;
    my $Shop = $Servlet->object();

    my $Form = $Servlet->form;
    my $hValues = $Form->form($Shop, 'ListedObjects');
    my @ListObjectIDs = map { $_->{'ListObjectID'} } @{$hValues->{'ListObjectIDs'}};

    # if the list contains the SuperMerchant, it is not allowed to delete the Users
    my $SuperMerchantID = $Shop->get('SuperMerchant')->id;
    Error('CANNOT_DELETE_SUPERMERCHANT', {
        'SuperMerchant'  => $SuperMerchantID,
        'ShopID' => $Shop->id,
    }) if ElementOf($SuperMerchantID, @ListObjectIDs);

    # if you are not the SuperMerchant, you are not allowed to delete other Merchants!
    my $CurrentUser = $Servlet->user;
    if(not $CurrentUser->get('IsSuperMerchant') ){
        my %MerchantIDs = map { $_->id => 1 } @{ $Shop->child('Groups')->child('Merchants')->get('Members') };
        foreach my $UserID (@ListObjectIDs){
            Error('MERCHANT_CANNOT_DELETE_MERCHANT', {
                'Merchant'  => $UserID,
                'ShopID' => $Shop->id,
            }) if exists $MerchantIDs{$UserID};
        }
    }

    $self->DeleteListedObjects( $Servlet );
    return;
}

sub SearchLastRegistrations {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object;

    $self->BlockMonitor($Servlet);

    my $SearchDefinition = GetSearchDefinition(LoadClassByAlias('User'), 'Users');
    my %Options = (
        'ObjectID'  => $Shop->id,
        'SearchDefinition' => $SearchDefinition,
        'Parameters' => {
            'Shop.ID'    => $Shop->id,
            'LanguageID' => $Servlet->languageID,
            'ExcludeGroupIDs' => [ map { $_->id } @{$Shop->get('NonStorefrontUserGroups')} ],
            'UserClass.ID' => LoadClassByAlias('User')->id,
        },
        'OrderBy'    => 'CreationDate',
        'OrderDesc'  => '1',
        'UriParameters'  => {
            'Search' => 'Users',
        },
        'Page' => 1,
        'PageSize' => 5,
    );
    LogDebug('Parameters', $Options{'Parameters'});

    my $Search = DE_EPAGES::Search::API::Object::PagerSearch->new(\%Options);
    $Search->statistic;
    return $Search->page;

}

sub SaveNumberFormats {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object();
    my $Form = $Servlet->form;

    my %HookValues = (
        'Shop' => $Shop,
        'Form' => $Form,
        'Values' => {},
    );
    TriggerHook('UI_SaveNumberFormats', \%HookValues);
    $Servlet->form->executeFormError();

    my $hValues = $HookValues{'Values'};

    #Set shop attributes: Next<ClassName>No
    #Despite perl critics complaint map is -not- used in void, but in list context.. inderctly.
    $Shop->set({ map { $_ => delete $hValues->{$_} } grep { /Next\w+No/ } keys %$hValues }); ## no critic qw(ProhibitVoidMap)

    #Set shop attribute: NumberFormats
    my %Values;
    foreach my $key (keys %$hValues){
        my ($fl,$sl) = split(/_/,$key);
        $Values{$fl}{$sl} = $hValues->{$key} if $hValues->{$key};
    }
    $Shop->set({ 'NumberFormats' => \%Values });

    return;
}

sub SetNextStepsItemDone  {
    my $self = shift;
    my ($Servlet) = @_;

    my $Item = $Servlet->form->value('Item');
    TestString('Item', $Item);
    $Item =~ s/[^\w]//igx;

    my $Done = $Servlet->form->value('Done');
    TestInteger('Done', $Done, 0, 1);

    my $Shop = $Servlet->object()->getSite;
    my $hItems = $Shop->get('NextSteps') // {};
    $hItems->{$Item} = $Done;
    $Shop->set({ 'NextSteps' => $hItems });

    return;
}

# called by clicking the "See your site" button, if the merchant
# has closed the shop
sub SetAdminCookie {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object();

    # crypted admin session by post param
    my $CryptedAdminSession = $Servlet->form->value('MerchantSFSession');
    my $PreviewStyle = $Servlet->form->value('PreviewStyle');
    my $IsMobile = $Servlet->form->value('IsMobile');
    my $WebServerScriptNameMobileStorefront = WebServerScriptNameMobileStorefront(GetUrlInfo($Shop));
    my $WebServerScriptNamePreview = WebServerScriptNamePreview(GetUrlInfo($Shop));
    my $WebServerScriptName = $Shop->get('WebServerScriptName');
    TestString('MerchantSFSession', $CryptedAdminSession);

    # set correct path for cookie
    # for mobile sf we do not have previews
    my $Path;
    if ($IsMobile) {
       $Path = $WebServerScriptNameMobileStorefront;
    } else {
        $Path = defined $PreviewStyle ? $WebServerScriptNamePreview : $WebServerScriptName;
    }

    #WebServerScriptUrl
    my %Options = (
        'Path'   => $Path,
        'Comment'=> 'for accessing the closed storefront',
        'HttpOnly' => 1,
    );

    $Servlet->cookies('MerchantSF', $CryptedAdminSession, \%Options);

    return;
}
# deletes an admin cookie for shop for showing the closed shop sf
sub DeleteAdminCookie {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object();

    my $WebServerScriptName = $Shop->get('WebServerScriptName');
    SetCookie( $Servlet->response(), 'MerchantSF', undef, {'Path'=> $WebServerScriptName} );

}

# ViewAction is called, if the merchant would like to go to the closed sf,
# after ChangeAction SetAdminCookie
sub ViewMerchantSF {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object();

    my $PreviewStyle = $Servlet->form->value('PreviewStyle');
    # redirect to the storefront here
    my $url = $PreviewStyle ? $Shop->get('WebServerScriptPreviewUrlSSL') : $Shop->get('WebServerScriptUrlSSL');

    #add object path if needed
    my $SFViewParams = $Servlet->form->value('SFViewParams');
    $url .=  "?ObjectPath=$SFViewParams" if $SFViewParams;

    # test if we have a valid url
    eval { TestUrl('MerchantSFRedirectURL', $url); };
    if ( ExistsError() ) {
        #wrong URL format
        my $Error = GetError();
        $Servlet->handleWrongUrlParameter( $Error );
    } else {
        $Servlet->vars('ExitAfterEvent', 1);
        $Servlet->redirect( $url,HTTP_SEE_OTHER);
    }
}

sub SaveUnityGeneralSettings {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;
    my $ShopAddress = $Shop->get('Address');
    my $Form = $Servlet->form;

    # check form values for shop without executing the error
    $Form->form($Shop, 'UnityGeneralSettings', 1);
    # check form values for the address with executing the errors
    my $hAddressData = $Form->form($ShopAddress, 'AddressDataUnity', 1);

    $self->attributeValues( $Servlet, $ShopAddress, $ShopAddress->class );

    # update shop
    eval {
        $self->SUPER::Save($Servlet);
    };
    if (ExistsError()) {
        my $Error = GetError();
        if ($Error->code eq 'UNITY_API_CALL_ERROR') {
            LogError($Error);
            $Form->executeFormError({
                'Reason'=> 'UNITY_API_CALL_ERROR',
                'Name'  => 'Shop',
                'Form'  => 'UnityGeneralSettings',
                'GUID'  => $Error->guid
            });
        } else {
            Error($Error);
        }
    }

    # update shop address
    $ShopAddress->set($hAddressData);

    return;
}

sub ViewShopSurvey {
    my $self = shift;
    my ($Servlet) = @_;

}

sub View {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object();
    my $Form = $Servlet->form;

    my %HookValues = (
        'Servlet' => $Servlet,
        'Shop' => $Shop,
        'Form' => $Form,
        'Values' => {},
    );

    TriggerHook('UI_MBOViewMBO',\%HookValues);

    # view shop
    $self->SUPER::View($Servlet);

}

sub UnityViewStatusGeneral {
    my $self = shift;
    my ($Servlet) = @_;

    # enable language switch
    $Servlet->vars('ShowLanguageSwitch', 1);

    # trigger hook to allow the adding of context sensitive apps
    TriggerHook('UI_Shop_UnityViewStatusGeneral', { 'Servlet' => $Servlet });

    $self->SUPER::View($Servlet);
}

#========================================================================================
# function     UnitySaveEditorHeaderSettings
# state        public
#----------------------------------------------------------------------------------------
# example      ep.ajax({
#                 type: "post",
#                 dataType: "json",
#                 data: {
#                   ViewAction: "JSONViewResponse",
#                   ChangeAction: "UnitySaveEditorHeaderSettings",
#                   logoUrl: e.data.payload.logoUrl,
#                   shouldHideLogo: e.data.payload.shouldHideLogo | 0
#                 }
#               })
#----------------------------------------------------------------------------------------
# description  Recieves an AJAX request with editor header settings, when header
#               settings get submitted in editor
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet with form data | object
#========================================================================================
sub UnitySaveEditorHeaderSettings {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;
    my $LogoUrl = $Servlet->form->value('logoUrl');
    my $ShouldHideLogo = $Servlet->form->value('shouldHideLogo');

    unless ($LogoUrl =~ /image\?remote=/) {
        $LogoUrl .= '&shop=' . GetUnitySubDomain($Shop->get('WebServerScriptNamePart'));
    }

    $Shop->set({'Logo' => $LogoUrl, 'ShouldHideLogo' => $ShouldHideLogo});
}

#========================================================================================
# function     UnitySaveEditorStorefrontSettings
# state        public
#----------------------------------------------------------------------------------------
# example      ep.ajax({
#                 type: "post",
#                 dataType: "json",
#                 data: {
#                   ViewAction: "JSONViewResponse",
#                   ChangeAction: "UnitySaveEditorStorefrontSettings",
#                   faviconUrl: e.data.payload.faviconUrl,
#                 }
#               })
#----------------------------------------------------------------------------------------
# description  Recieves an AJAX request with editor storefront settings, when storefront
#               settings get submitted in editor
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet with form data | object
#========================================================================================
sub UnitySaveEditorStorefrontSettings {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;
    my $FaviconUrl = $Servlet->form->value('faviconUrl');

    $Shop->set({'FavIcon' => $FaviconUrl});
}

1;
