#========================================================================================
# package      DE_EPAGES::ShopMigration::API::TLE::ShopMigrationHandler
# state        public
# description  TLE functions for Migration Dialog
#========================================================================================
package DE_EPAGES::ShopMigration::API::TLE::ShopMigrationHandler;

use strict;
use LWP::UserAgent;
use JSON;
use DE_EPAGES::Core::API::TestParameter qw ( TestObject );
use DE_EPAGES::Database::API::Connection qw (GetCurrentDBHandle);
use DE_EPAGES::ShopMigration::API::Constants qw ( MIGRATION_DIALOG_REPEAT_AFTER_DAYS );
use DE_EPAGES::XML::API::SOAPDateTimeFormatXML;

#========================================================================================
# function     RegisterHandlerProc
# state        private
#----------------------------------------------------------------------------------------
# syntax       $RegisterHandlerProc( $hParams );
#----------------------------------------------------------------------------------------
# description  this function registers this attribute handle on the processor via hook
#               function on hook TLEProcessorRegistration
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments keys :
#               <ul>
#                   <li>Processor - tle processor - object
#               </ul> | ref.hash
#========================================================================================
sub RegisterHandlerProc {
    my ($Params) = @_;

    __PACKAGE__->new()->register( $Params->{'Processor'} );
    return;
}

#========================================================================================
# function     new
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler = new DE_EPAGES::Object::API::TLE::ShopMigrationHandler;
# example      my $PrefixHandler  = new DE_EPAGES::Object::API::TLE::ShopMigrationHandler;
#----------------------------------------------------------------------------------------
# description  create a new handler
#----------------------------------------------------------------------------------------
# return       $Handler | blessed object | object
#========================================================================================
sub new { bless {}, shift; }

#========================================================================================
# function     register
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler->register( $Processor );
#----------------------------------------------------------------------------------------
# description  this function register this handle on the processor
#----------------------------------------------------------------------------------------
# input        $Processor | tle processor | object
#========================================================================================
sub register {
    my $self = shift;
    my ($Processor) = @_;

    $Processor->registerHandler('FunctionHandler', $self, 'MigrationDialog');
    return;
}

#========================================================================================
# function     MigrationDialog
# state        public
#----------------------------------------------------------------------------------------
# syntax       #LOCAL("MyMigrationDialog",#MigrationDialog(#Shop.Object) )
#                   #MyMigrationDialog.OpenDialog
#               #ENDLOCAL
#----------------------------------------------------------------------------------------
# description  Calculates parameters for the migration dialog and write received data to the TLE
#               #OpenDialog #IsNotInterested #DisplayCount #DialogCloseDate.
#----------------------------------------------------------------------------------------
# input        $Processor | tle processor | object
# input        $aParams | List of Parameters | array ref
# return       $Content | TLEs | string
#========================================================================================
sub MigrationDialog {
    my $self = shift;
    my ($Processor, $aParameter) = @_;

    my ($Shop) = @$aParameter;
    TestObject('Shop', $Shop, 'DE_EPAGES::Shop::API::Object::Shop');

    my $hVars = {
        'OpenDialog' => 0,
        'IsNotInterested' => 0,
        'DialogCount' => 0,
        'PrepShopDialogSeen' => 0,
        'DialogCloseDate' => undef
    };

    #TODO:  use attributes from ShopMigration Cartridge for testing if we have a shop for migrating to Now
    my $hShopMigrParams = $Shop->get(['IsUnityRegistered', 'IsMigrationShop']);

    # we do nothing if we have already a Now shop or the shop is not migrateable
    return $hVars if ( $hShopMigrParams->{'IsUnityRegistered'} or (not $hShopMigrParams->{'IsMigrationShop'}) );

    # get already written parameters from the attribute MigrationDialogParameters
    my $JSONDialogParameters = $Shop->get('MigrationDialogParameters');

    # we have no saved parameters from the dialog (initial call)
    unless ($JSONDialogParameters) {
        $hVars->{'OpenDialog'} = 1;
        return $hVars;
    }

    # we will show the dialog again after MIGRATION_DIALOG_REPEAT_AFTER_DAYS days after closing the dialog
    # and only if the parameter IsNotInterested is not set
    # extract parameters DialogCloseDate
    my $hDialogParameters = decode_json($JSONDialogParameters);
    $hDialogParameters->{'OpenDialog'} = 0;

    # not interested
    return $hDialogParameters if $hDialogParameters->{'IsNotInterested'} eq '1';

    # test date time: repeat after MIGRATION_DIALOG_REPEAT_AFTER_DAYS days
    my $Formatter = DE_EPAGES::XML::API::SOAPDateTimeFormatXML->new();
    my $DialogCloseDate = $Formatter->parse_datetime($hDialogParameters->{'DialogCloseDate'});

    $DialogCloseDate->add('days' => MIGRATION_DIALOG_REPEAT_AFTER_DAYS);
    $hDialogParameters->{'OpenDialog'} = 1 if (GetCurrentDBHandle()->currentDateTime > $DialogCloseDate );

    return $hDialogParameters;
}

1;
