#==============================================================================
# package      DE_EPAGES::ShopMigration::UI::Shop
# state        private
#==============================================================================
package DE_EPAGES::ShopMigration::UI::Shop;
use base DE_EPAGES::Presentation::UI::Object;

use strict;
use HTTP::Status qw(RC_SEE_OTHER);
use DE_EPAGES::Core::API::Error qw(Error ExistsError GetError);
use DE_EPAGES::Core::API::PerlTools qw(GenerateGUID GetCurrentDateTime);
use DE_EPAGES::Core::API::Warning qw(Warning);
use DE_EPAGES::Object::API::Factory qw(LoadClassByAlias);
use DE_EPAGES::ShopMigration::API::Constants qw (SHOPMIGRATION_FINAL_ALLOW SHOPMIGRATION_ABORTED);
use DE_EPAGES::ShopMigration::API::PrepShopLoginToken qw(IsValidToken);

sub ErrorDefinition {
    return {
        'NO_PREPSHOP'                   => 'No preparation shop found.',
        'NO_PREPSHOP_ADMIN'             => 'No supermerchant found.',
        'PREPSHOP_INVALID_LOGIN'        => 'Login name is not valid.',
        'PREPSHOP_INVALID_TOKEN'        => 'Token is not valid.',
        'NO_MIGRATIONSHOP_FOR_PREPSHOP' => 'No migration shop for preparation shop found.',
    };
}

#==============================================================================
# Event handlers
#==============================================================================
sub PreparePrepShopLoginRedirect {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object();
    my $PrepShop = $Shop->get('MigrationPrepShop');
    Error('NO_PREPSHOP') unless $PrepShop;

    my $PrepShopAdmin = $PrepShop->get('SuperMerchant');
    Error('NO_PREPSHOP_ADMIN') unless $PrepShopAdmin;

    # generate and save login token
    my $LoginToken = GenerateGUID();
    my $hInfoLoginToken = {
        'Alias'   => $LoginToken,
        'IsValid' => 1,
        'User'    => $PrepShopAdmin,
    };
    my $TokenClass = LoadClassByAlias('PrepShopLoginToken');
    $PrepShopAdmin->folder('PrepShopLoginTokens')->insertChild($TokenClass, $hInfoLoginToken);

    $Servlet->vars('LoginRedirectUrl', $PrepShop->get('WebUrlAdminSSL'));
    $Servlet->vars('LoginToken', $LoginToken);
    $Servlet->vars('LoginName', $PrepShopAdmin->alias);
    $Servlet->vars('LoginChangeAction', 'MBO-PrepShopTokenLogin');
    $Servlet->vars('ViewAction', 'MBO-ViewPrepShopLoginRedirect');
    $Servlet->vars('ObjectID', $Shop->id);

}

sub MBOPrepShopTokenLogin {
    my $self = shift;
    my ($Servlet) = @_;

    my $LoginToken = $Servlet->params('LoginToken');
    my $LoginName = $Servlet->params('LoginName');

    my $PrepShop = $Servlet->object;
    my $Merchant = $PrepShop->get('SuperMerchant');

    Error('PREPSHOP_INVALID_LOGIN') if $Merchant->alias ne $LoginName;

    my $TokenFolder = $Merchant->folder('PrepShopLoginTokens');

    # check if token is valid
    if (IsValidToken($LoginToken, $TokenFolder)) {

        # Login merchant
        $Servlet->registerSession($Merchant, 1);

        my $Response = $Servlet->response;
        $Response->code(RC_SEE_OTHER); # 303 -> means See Other
        $Response->header('Location', $PrepShop->get('WebUrlAdminSSL'));

        # set the shopattribute LastMerchantLogin
        eval {
            $PrepShop->set({ 'LastMerchantLogin' => GetCurrentDateTime() });
        };

        # error is caused by long running process which blocks shop table (EPC-234)
        if (ExistsError()) {
            my $Error = GetError();
            my $ErrorCode = $Error->code;
            my $ErrorMsg = $Error->shortMessage;
            if ($ErrorCode eq 'DBIExecute' && $ErrorMsg =~ /Lock wait timeout exceeded/) {
                Warning($Error);
            }
            else {
                Error($Error);
            }
        }

        # Invalidate token
        $TokenFolder->child($LoginToken)->set({ 'IsValid' => 0 });
    }
    else {
        Error('PREPSHOP_INVALID_TOKEN');
    }
    $Servlet->vars('ExitAfterEvent', 1);
    return;
}

sub PrepShopStartShopUpdate {
    my $self = shift;
    my ($Servlet) = @_;
    my $PrepShop = $Servlet->object;
    Error('NO_PREPSHOP') unless $PrepShop->get('IsMigrationPrepShop');

    my $MigrationShop = $PrepShop->get('MigrationShop');
    Error('NO_MIGRATIONSHOP_FOR_PREPSHOP') unless defined $MigrationShop;

    $MigrationShop->set({
            'IsClosed'=> 1,
            'MigrationStatus' => SHOPMIGRATION_FINAL_ALLOW
    });
    $Servlet->logout();
    return;
}

sub AbortShopMigration {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object;
    Error('NO_MIGRATIONSHOP') unless $Shop->get('IsMigrationShop');

    my $PrepShop = $Shop->get('MigrationPrepShop');
    Error('NO_PREPSHOP') unless defined $PrepShop;

    $Shop->set({
        'MigrationStatus' => SHOPMIGRATION_ABORTED
    });
    return;
}

1;
