#========================================================================================
# §package      DE_EPAGES::ShopMigration::t::ProductsExportImport
# §base         DE_EPAGES::Test::API::UnitTest::Test
#----------------------------------------------------------------------------------------
# §description  unit test for DE_EPAGES::ShopMigration::API::JSONLines::ProductsExportImport
#========================================================================================
package DE_EPAGES::ShopMigration::t::ProductsExportImport;
use base DE_EPAGES::Test::API::StoreUnitTest;
## no critic (Tics::ProhibitLongLines)
use strict;
use utf8;
use warnings;
use File::Spec::Functions;
use Storable qw(dclone);
use DE_EPAGES::Core::API::File qw(GetFileContent GetFileContentLines ExistsFile DeleteDirectory DeleteFile Text2Unix GetFileNames);
use DE_EPAGES::Object::API::Factory qw(LoadRootObject LoadClassByAlias ResetFactoryCaches LoadObjectByPath);
use DE_EPAGES::Object::API::Language qw(GetPKeyLanguageByCode);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw(ImportObjects);
use DE_EPAGES::ShopMigration::API::JSONLines::BaseExportDriver;
use DE_EPAGES::ShopMigration::API::JSONLines::ProductsExportImport qw(ExportProductsJSON ImportProductsJSON);
use DE_EPAGES::ShopMigration::API::MigrationShop qw(CreatePrepShop ExistsPrepShop SetShopFeatureValues);
use DE_EPAGES::ShopMigration::API::ProductsCommon qw(GetProductData);
use DE_EPAGES::ShopMigration::API::ResetShopHandler;
use File::Temp qw(tempdir);
use Test::More;
use Test::Differences qw(eq_or_diff);

my $TmpDir = tempdir();

if ($ENV{SKIP_IN_PR_TESTS}) {
    plan skip_all => 'Skipping it because SKIP_IN_PR_TESTS == 1';
} else {
    plan tests => 42;
}

# hack BaseExportdDriver to use canonical for easier diff, not enabled per default for performance reasons
$DE_EPAGES::ShopMigration::API::JSONLines::BaseExportDriver::JSON_OPTS{canonical} = 1;

sub test {
    testExportWithProductsExportImportBase2Prep();
    testExportWithProductsExportImportPrep2Base();
    testImportWithProductsExportImportBase2Prep();
    testImportWithProductsExportImportPrep2Base();
    done_testing();
}

my %TestData = (
    TestShop      => undef,
    TestShopAlias => 'TestShop',
    ImportPath    => '/Test/Shops/TestShop',
    OldValues     => {
        '1001'      => {
            'Attributes'          => {
                AvailabilityDate                    => undef,
                'ExpiryDate'                        => undef,
                'EnergyLabelHighestEfficiencyClass' => undef,
                'HasSubOwnPrices'                   => '1',
                'RelativeFilePathPart'              => 'Products/1001',
                'PriceQuantity'                     => '1',
                'ImageHotDeal'                      => 'frame_h.jpg',
                'EcoParticipationCategory'          => undef,
                'TaxClass'                          => 'normal',
                'ImageLarge'                        => 'frame.jpg',
                'Manufacturer'                      => undef,
                'EnergyLabelsString'                => undef,
                'OrderUnit'                         => '/Units/piece',
                'RefContentAmount'                  => '1',
                'IsDefault'                         => '0',
                'MinOrder'                          => '1',
                'Position'                          => '10',
                'RefAmount'                         => '1',
                'Height'                            => undef,
                'IsDailyPrice'                      => '0',
                'RefUnit'                           => '/Units/byte',
                'ImageMedium'                       => 'frame_m.jpg',
                'NewnessDate'                       => undef,
                'RatingAverage'                     => undef,
                'ListPrices'                        => [
                    {
                        'ManufacturerPrice' => undef,
                        'CurrencyID'        => 'EUR',
                        'EcoParticipation'  => undef,
                        'ListPrice'         => '38.89',
                        'Deposit'           => undef
                    }
                ],
                'TaxClassDigital'                   => undef,
                'Class'                             => 'MyProductType',
                'DeliveryPeriod'                    => '3',
                'ImageMediumSmall'                  => 'frame_ms.jpg',
                'ImagesSlideShowString'             => 'frame.jpg;frame-1.jpg;frame-2.jpg',
                'IsAvailable'                       => '1',
                'ManufacturerSKU'                   => undef,
                'Width'                             => undef,
                'ImageSmall'                        => 'frame_s.jpg',
                'ImageMediumLarge'                  => 'frame_ml.jpg',
                'IsNew'                             => '0',
                'EnergyLabelSourceFile'             => undef,
                'IsVisible'                         => '1',
                'WeightUnit'                        => '/Units/gram',
                'Weight'                            => '1',
                'UPCEAN'                            => undef,
                'Alias'                             => '1001',
                'Length'                            => undef,
                'MyPreDefLocalizedString'           => 'blau',
                'ImageThumbnail'                    => undef,
                'IntervalOrder'                     => '1',
                'SuperProduct'                      => undef
            },
            'LocalizedAttributes' => {
                'en' => {
                    'AvailabilityComment'  => undef,
                    'MainCharacteristics'  => 'Rahmen: Schwarz',
                    'Title'                => undef,
                    'Text'                 => 'Das ist alles Meta',
                    '_ProductDataSheet'    => 'IT_20200422_005933.pdf',
                    'Keywords'             => 'Kunst, Gemälde, Deko',
                    'MyDate'               => '24/07/2020',
                    'MyDateTime'           => '24/07/2020 11:30',
                    'MyFloat'              => '1.5',
                    'MyInteger'            => '11',
                    'MyMoney'              => '5.55 €',
                    'MyString'             => 'someString',
                    'MyTime'               => '12:12',
                    'Name'                 => '[Beispiel] Gerahmte Zeichnung',
                    'Description'          => '<p>Diese detaillierte Zeichnung ist ein Hingucker für jedes Zuhause. Die Zeichnung wird in einem hochwertigen, schwarzen Holzrahmen geliefert. Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua.</p>',
                    'ProductVideoProvider' => undef
                }
            }
        },
        '1001-0001' => {
            'Attributes'          => {
                AvailabilityDate                    => undef,
                'ExpiryDate'                        => undef,
                'EnergyLabelHighestEfficiencyClass' => undef,
                'HasSubOwnPrices'                   => '1',
                'RelativeFilePathPart'              => 'Products/1001',
                'PriceQuantity'                     => '1',
                'ImageHotDeal'                      => 'frame_h.jpg',
                'EcoParticipationCategory'          => undef,
                'TaxClass'                          => 'normal',
                'ImageLarge'                        => 'frame.jpg',
                'Manufacturer'                      => undef,
                'EnergyLabelsString'                => undef,
                'OrderUnit'                         => '/Units/piece',
                'RefContentAmount'                  => '1',
                'IsDefault'                         => '0',
                'MinOrder'                          => '1',
                'Position'                          => '10',
                'RefAmount'                         => '1',
                'Height'                            => undef,
                'IsDailyPrice'                      => '0',
                'RefUnit'                           => '/Units/byte',
                'ImageMedium'                       => 'frame_m.jpg',
                'NewnessDate'                       => undef,
                'RatingAverage'                     => undef,
                'ListPrices'                        => [
                    {
                        'ManufacturerPrice' => undef,
                        'CurrencyID'        => 'EUR',
                        'EcoParticipation'  => undef,
                        'ListPrice'         => '38.89',
                        'Deposit'           => undef
                    }
                ],
                'TaxClassDigital'                   => undef,
                'Class'                             => 'MyProductType',
                'DeliveryPeriod'                    => '3',
                'ImageMediumSmall'                  => 'frame_ms.jpg',
                'ImagesSlideShowString'             => 'frame.jpg;frame-1.jpg;frame-2.jpg',
                'IsAvailable'                       => '1',
                'ManufacturerSKU'                   => undef,
                'Width'                             => undef,
                'ImageSmall'                        => 'frame_s.jpg',
                'ImageMediumLarge'                  => 'frame_ml.jpg',
                'IsNew'                             => '0',
                'EnergyLabelSourceFile'             => undef,
                'IsVisible'                         => '1',
                'WeightUnit'                        => '/Units/gram',
                'Weight'                            => '1',
                'UPCEAN'                            => undef,
                'Alias'                             => '1001-0001',
                'Length'                            => undef,
                'MyPreDefLocalizedString'           => 'blau',
                'ImageThumbnail'                    => undef,
                'IntervalOrder'                     => '1',
                'SuperProduct'                      => 'Products/1001'
            },
            'LocalizedAttributes' => {
                'en' => {
                    'AvailabilityComment'  => undef,
                    'MainCharacteristics'  => 'Rahmen: Schwarz',
                    'Title'                => undef,
                    'Text'                 => 'Das ist alles Meta',
                    '_ProductDataSheet'    => 'IT_20200422_005933.pdf',
                    'Keywords'             => 'Kunst, Gemälde, Deko',
                    'MyDate'               => '24/07/2020',
                    'MyDateTime'           => '24/07/2020 11:30',
                    'MyFloat'              => '1.5',
                    'MyInteger'            => '11',
                    'MyMoney'              => '5.55 €',
                    'MyString'             => 'someString',
                    'MyTime'               => '12:12',
                    'Name'                 => '[Beispiel] Gerahmte Zeichnung',
                    'Description'          => '<p>Diese detaillierte Zeichnung ist ein Hingucker für jedes Zuhause. Die Zeichnung wird in einem hochwertigen, schwarzen Holzrahmen geliefert. Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua.</p>',
                    'ProductVideoProvider' => undef
                }
            }
        },
        '1001-0002' => {
            'Attributes'          => {
                AvailabilityDate                    => undef,
                'ExpiryDate'                        => undef,
                'EnergyLabelHighestEfficiencyClass' => undef,
                'HasSubOwnPrices'                   => '1',
                'RelativeFilePathPart'              => 'Products/1001',
                'PriceQuantity'                     => '1',
                'ImageHotDeal'                      => 'frame_h.jpg',
                'EcoParticipationCategory'          => undef,
                'TaxClass'                          => 'normal',
                'ImageLarge'                        => 'frame.jpg',
                'Manufacturer'                      => undef,
                'EnergyLabelsString'                => undef,
                'OrderUnit'                         => '/Units/piece',
                'RefContentAmount'                  => '1',
                'IsDefault'                         => '0',
                'MinOrder'                          => '1',
                'Position'                          => '20',
                'RefAmount'                         => '1',
                'Height'                            => undef,
                'IsDailyPrice'                      => '0',
                'RefUnit'                           => '/Units/byte',
                'ImageMedium'                       => 'frame_m.jpg',
                'NewnessDate'                       => undef,
                'RatingAverage'                     => undef,
                'ListPrices'                        => [
                    {
                        'ManufacturerPrice' => undef,
                        'CurrencyID'        => 'EUR',
                        'EcoParticipation'  => undef,
                        'ListPrice'         => '38.89',
                        'Deposit'           => undef
                    }
                ],
                'TaxClassDigital'                   => undef,
                'Class'                             => 'MyProductType',
                'DeliveryPeriod'                    => '3',
                'ImageMediumSmall'                  => 'frame_ms.jpg',
                'ImagesSlideShowString'             => 'frame.jpg;frame-1.jpg;frame-2.jpg',
                'IsAvailable'                       => '1',
                'ManufacturerSKU'                   => undef,
                'Width'                             => undef,
                'ImageSmall'                        => 'frame_s.jpg',
                'ImageMediumLarge'                  => 'frame_ml.jpg',
                'IsNew'                             => '0',
                'EnergyLabelSourceFile'             => undef,
                'IsVisible'                         => '1',
                'WeightUnit'                        => '/Units/gram',
                'Weight'                            => '1',
                'UPCEAN'                            => undef,
                'Alias'                             => '1001-0002',
                'Length'                            => undef,
                'MyPreDefLocalizedString'           => 'green',
                'ImageThumbnail'                    => undef,
                'IntervalOrder'                     => '1',
                'SuperProduct'                      => 'Products/1001'
            },
            'LocalizedAttributes' => {
                'en' => {
                    'AvailabilityComment'  => undef,
                    'MainCharacteristics'  => 'Rahmen: Schwarz',
                    'Title'                => undef,
                    'Text'                 => 'Das ist alles Meta',
                    '_ProductDataSheet'    => 'IT_20200422_005933.pdf',
                    'Keywords'             => 'Kunst, Gemälde, Deko',
                    'MyDate'               => '24/07/2020',
                    'MyDateTime'           => '24/07/2020 11:30',
                    'MyFloat'              => '1.5',
                    'MyInteger'            => '11',
                    'MyMoney'              => '5.55 €',
                    'MyString'             => 'someString',
                    'MyTime'               => '12:12',
                    'Name'                 => '[Beispiel] Gerahmte Zeichnung',
                    'Description'          => '<p>Diese detaillierte Zeichnung ist ein Hingucker für jedes Zuhause. Die Zeichnung wird in einem hochwertigen, schwarzen Holzrahmen geliefert. Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua.</p>',
                    'ProductVideoProvider' => undef
                }
            }
        },
        '1001-0003' => {
            'Attributes'          => {
                AvailabilityDate                    => undef,
                'ExpiryDate'                        => undef,
                'EnergyLabelHighestEfficiencyClass' => undef,
                'HasSubOwnPrices'                   => '1',
                'RelativeFilePathPart'              => 'Products/1001',
                'PriceQuantity'                     => '1',
                'ImageHotDeal'                      => 'frame_h.jpg',
                'EcoParticipationCategory'          => undef,
                'TaxClass'                          => 'normal',
                'ImageLarge'                        => 'frame.jpg',
                'Manufacturer'                      => undef,
                'EnergyLabelsString'                => undef,
                'OrderUnit'                         => '/Units/piece',
                'RefContentAmount'                  => '1',
                'IsDefault'                         => '0',
                'MinOrder'                          => '1',
                'Position'                          => '30',
                'RefAmount'                         => '1',
                'Height'                            => undef,
                'IsDailyPrice'                      => '0',
                'RefUnit'                           => '/Units/byte',
                'ImageMedium'                       => 'frame_m.jpg',
                'NewnessDate'                       => undef,
                'RatingAverage'                     => undef,
                'ListPrices'                        => [
                    {
                        'ManufacturerPrice' => undef,
                        'CurrencyID'        => 'EUR',
                        'EcoParticipation'  => undef,
                        'ListPrice'         => '38.89',
                        'Deposit'           => undef
                    }
                ],
                'TaxClassDigital'                   => undef,
                'Class'                             => 'MyProductType',
                'DeliveryPeriod'                    => '3',
                'ImageMediumSmall'                  => 'frame_ms.jpg',
                'ImagesSlideShowString'             => 'frame.jpg;frame-1.jpg;frame-2.jpg',
                'IsAvailable'                       => '1',
                'ManufacturerSKU'                   => undef,
                'Width'                             => undef,
                'ImageSmall'                        => 'frame_s.jpg',
                'ImageMediumLarge'                  => 'frame_ml.jpg',
                'IsNew'                             => '0',
                'EnergyLabelSourceFile'             => undef,
                'IsVisible'                         => '1',
                'WeightUnit'                        => '/Units/gram',
                'Weight'                            => '1',
                'UPCEAN'                            => undef,
                'Alias'                             => '1001-0003',
                'Length'                            => undef,
                'MyPreDefLocalizedString'           => 'white',
                'ImageThumbnail'                    => undef,
                'IntervalOrder'                     => '1',
                'SuperProduct'                      => 'Products/1001'
            },
            'LocalizedAttributes' => {
                'en' => {
                    'AvailabilityComment'  => undef,
                    'MainCharacteristics'  => 'Rahmen: Schwarz',
                    'Title'                => undef,
                    'Text'                 => 'Das ist alles Meta',
                    '_ProductDataSheet'    => 'IT_20200422_005933.pdf',
                    'Keywords'             => 'Kunst, Gemälde, Deko',
                    'MyDate'               => '24/07/2020',
                    'MyDateTime'           => '24/07/2020 11:30',
                    'MyFloat'              => '1.5',
                    'MyInteger'            => '11',
                    'MyMoney'              => '5.55 €',
                    'MyString'             => 'someString',
                    'MyTime'               => '12:12',
                    'Name'                 => '[Beispiel] Gerahmte Zeichnung',
                    'Description'          => '<p>Diese detaillierte Zeichnung ist ein Hingucker für jedes Zuhause. Die Zeichnung wird in einem hochwertigen, schwarzen Holzrahmen geliefert. Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua.</p>',
                    'ProductVideoProvider' => undef
                }
            }
        },
    },
    NewValues     => {
        '1001'      => {
            'Attributes'          => {
                AvailabilityDate                    => undef,
                'EnergyLabelHighestEfficiencyClass' => undef,
                'ExpiryDate'                        => undef,
                'HasSubOwnPrices'                   => '1',
                'RelativeFilePathPart'              => 'Products/1001',
                'PriceQuantity'                     => '1',
                'ImageHotDeal'                      => 'frame_h.jpg',
                'EcoParticipationCategory'          => undef,
                'TaxClass'                          => 'normal',
                'ImageLarge'                        => 'frame.jpg',
                'Manufacturer'                      => undef,
                'EnergyLabelsString'                => undef,
                'OrderUnit'                         => '/Units/piece',
                'RefContentAmount'                  => '1',
                'IsDefault'                         => '0',
                'MinOrder'                          => '1',
                'Position'                          => '10',
                'RefAmount'                         => '1',
                'Height'                            => '3501',
                'IsDailyPrice'                      => '0',
                'RefUnit'                           => '/Units/byte',
                'ImageMedium'                       => 'frame_m.jpg',
                'NewnessDate'                       => undef,
                'RatingAverage'                     => undef,
                'ListPrices'                        => [
                    {
                        'ManufacturerPrice' => '35.01',
                        'CurrencyID'        => 'EUR',
                        'EcoParticipation'  => undef,
                        'ListPrice'         => '38.89',
                        'Deposit'           => undef
                    }
                ],
                'TaxClassDigital'                   => undef,
                'Class'                             => 'MyProductType',
                'DeliveryPeriod'                    => '3',
                'ImageMediumSmall'                  => 'frame_ms.jpg',
                'ImagesSlideShowString'             => 'frame.jpg;frame-1.jpg;frame-2.jpg',
                'IsAvailable'                       => '1',
                'ManufacturerSKU'                   => undef,
                'Width'                             => undef,
                'ImageSmall'                        => 'frame_s.jpg',
                'ImageMediumLarge'                  => 'frame_ml.jpg',
                'IsNew'                             => '0',
                'EnergyLabelSourceFile'             => undef,
                'IsVisible'                         => '1',
                'WeightUnit'                        => '/Units/gram',
                'Weight'                            => '1',
                'UPCEAN'                            => undef,
                'Alias'                             => '1001',
                'Length'                            => undef,
                'MyPreDefLocalizedString'           => 'blau',
                'ImageThumbnail'                    => undef,
                'IntervalOrder'                     => '1',
                'SuperProduct'                      => undef
            },
            'LocalizedAttributes' => {
                'en' => {
                    'MyString'             => 'someString',
                    'AvailabilityComment'  => undef,
                    'MainCharacteristics'  => 'Rahmen: Schwarz',
                    'MyDateTime'           => '24/07/2020 11:30',
                    'MyFloat'              => '1.5',
                    'MyTime'               => '12:12',
                    'Title'                => undef,
                    'Text'                 => undef,
                    '_ProductDataSheet'    => 'IT_20200422_005933.pdf',
                    'Keywords'             => 'Kunst, Gemälde, Deko',
                    'MyMoney'              => '5.55 €',
                    'MyDate'               => '24/07/2020',
                    'MyInteger'            => '11',
                    'Name'                 => '[Beispiel] Gerahmte Zeichnung',
                    'Description'          => 'Kurze Beschreibung Rahmen (alt)<p>Diese detaillierte Zeichnung ist ein Hingucker für jedes Zuhause. Die Zeichnung wird in einem hochwertigen, schwarzen Holzrahmen geliefert. Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua.</p>',
                    'ProductVideoProvider' => undef
                }
            }
        },
        '1001-0001' => {
            'Attributes'          => {
                AvailabilityDate                    => undef,
                'EnergyLabelHighestEfficiencyClass' => undef,
                'ExpiryDate'                        => undef,
                'HasSubOwnPrices'                   => '1',
                'RelativeFilePathPart'              => 'Products/1001',
                'PriceQuantity'                     => '1',
                'ImageHotDeal'                      => 'frame_h.jpg',
                'EcoParticipationCategory'          => undef,
                'TaxClass'                          => 'normal',
                'ImageLarge'                        => 'frame.jpg',
                'Manufacturer'                      => undef,
                'EnergyLabelsString'                => undef,
                'OrderUnit'                         => '/Units/piece',
                'RefContentAmount'                  => '1',
                'IsDefault'                         => '0',
                'MinOrder'                          => '1',
                'Position'                          => '10',
                'RefAmount'                         => '1',
                'Height'                            => '3501',
                'IsDailyPrice'                      => '0',
                'RefUnit'                           => '/Units/byte',
                'ImageMedium'                       => 'frame_m.jpg',
                'NewnessDate'                       => undef,
                'RatingAverage'                     => undef,
                'ListPrices'                        => [
                    {
                        'ManufacturerPrice' => '35.01',
                        'CurrencyID'        => 'EUR',
                        'EcoParticipation'  => undef,
                        'ListPrice'         => '38.89',
                        'Deposit'           => undef
                    }
                ],
                'TaxClassDigital'                   => undef,
                'Class'                             => 'MyProductType',
                'DeliveryPeriod'                    => '3',
                'ImageMediumSmall'                  => 'frame_ms.jpg',
                'ImagesSlideShowString'             => 'frame.jpg;frame-1.jpg;frame-2.jpg',
                'IsAvailable'                       => '1',
                'ManufacturerSKU'                   => undef,
                'Width'                             => undef,
                'ImageSmall'                        => 'frame_s.jpg',
                'ImageMediumLarge'                  => 'frame_ml.jpg',
                'IsNew'                             => '0',
                'EnergyLabelSourceFile'             => undef,
                'IsVisible'                         => '1',
                'WeightUnit'                        => '/Units/gram',
                'Weight'                            => '1',
                'UPCEAN'                            => undef,
                'Alias'                             => '1001-0001',
                'Length'                            => undef,
                'MyPreDefLocalizedString'           => 'blau',
                'ImageThumbnail'                    => undef,
                'IntervalOrder'                     => '1',
                'SuperProduct'                      => 'Products/1001'
            },
            'LocalizedAttributes' => {
                'en' => {
                    'AvailabilityComment'  => undef,
                    'MainCharacteristics'  => 'Rahmen: Schwarz',
                    'Title'                => undef,
                    'Text'                 => undef,
                    '_ProductDataSheet'    => 'IT_20200422_005933.pdf',
                    'Keywords'             => 'Kunst, Gemälde, Deko',
                    'MyDate'               => '24/07/2020',
                    'MyDateTime'           => '24/07/2020 11:30',
                    'MyFloat'              => '1.5',
                    'MyInteger'            => '11',
                    'MyMoney'              => '5.55 €',
                    'MyString'             => 'someString',
                    'MyTime'               => '12:12',
                    'Name'                 => '[Beispiel] Gerahmte Zeichnung',
                    'Description'          => 'Kurze Beschreibung Rahmen (alt)<p>Diese detaillierte Zeichnung ist ein Hingucker für jedes Zuhause. Die Zeichnung wird in einem hochwertigen, schwarzen Holzrahmen geliefert. Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua.</p>',
                    'ProductVideoProvider' => undef
                }
            }
        },
        '1001-0002' => {
            'Attributes'          => {
                AvailabilityDate                    => undef,
                'EnergyLabelHighestEfficiencyClass' => undef,
                'ExpiryDate'                        => undef,
                'HasSubOwnPrices'                   => '1',
                'RelativeFilePathPart'              => 'Products/1001',
                'PriceQuantity'                     => '1',
                'ImageHotDeal'                      => 'frame_h.jpg',
                'EcoParticipationCategory'          => undef,
                'TaxClass'                          => 'normal',
                'ImageLarge'                        => 'frame.jpg',
                'Manufacturer'                      => undef,
                'EnergyLabelsString'                => undef,
                'OrderUnit'                         => '/Units/piece',
                'RefContentAmount'                  => '1',
                'IsDefault'                         => '0',
                'MinOrder'                          => '1',
                'Position'                          => '20',
                'RefAmount'                         => '1',
                'Height'                            => '3501',
                'IsDailyPrice'                      => '0',
                'RefUnit'                           => '/Units/byte',
                'ImageMedium'                       => 'frame_m.jpg',
                'NewnessDate'                       => undef,
                'RatingAverage'                     => undef,
                'ListPrices'                        => [
                    {
                        'ManufacturerPrice' => '35.01',
                        'CurrencyID'        => 'EUR',
                        'EcoParticipation'  => undef,
                        'ListPrice'         => '38.89',
                        'Deposit'           => undef
                    }
                ],
                'TaxClassDigital'                   => undef,
                'Class'                             => 'MyProductType',
                'DeliveryPeriod'                    => '3',
                'ImageMediumSmall'                  => 'frame_ms.jpg',
                'ImagesSlideShowString'             => 'frame.jpg;frame-1.jpg;frame-2.jpg',
                'IsAvailable'                       => '1',
                'ManufacturerSKU'                   => undef,
                'Width'                             => undef,
                'ImageSmall'                        => 'frame_s.jpg',
                'ImageMediumLarge'                  => 'frame_ml.jpg',
                'IsNew'                             => '0',
                'EnergyLabelSourceFile'             => undef,
                'IsVisible'                         => '1',
                'WeightUnit'                        => '/Units/gram',
                'Weight'                            => '1',
                'UPCEAN'                            => undef,
                'Alias'                             => '1001-0002',
                'Length'                            => undef,
                'MyPreDefLocalizedString'           => 'green',
                'ImageThumbnail'                    => undef,
                'IntervalOrder'                     => '1',
                'SuperProduct'                      => 'Products/1001'
            },
            'LocalizedAttributes' => {
                'en' => {
                    'AvailabilityComment'  => undef,
                    'MainCharacteristics'  => 'Rahmen: Schwarz',
                    'Title'                => undef,
                    'Text'                 => undef,
                    '_ProductDataSheet'    => 'IT_20200422_005933.pdf',
                    'Keywords'             => 'Kunst, Gemälde, Deko',
                    'MyDate'               => '24/07/2020',
                    'MyDateTime'           => '24/07/2020 11:30',
                    'MyFloat'              => '1.5',
                    'MyInteger'            => '11',
                    'MyMoney'              => '5.55 €',
                    'MyString'             => 'someString',
                    'MyTime'               => '12:12',
                    'Name'                 => '[Beispiel] Gerahmte Zeichnung',
                    'Description'          => 'Kurze Beschreibung Rahmen (alt)<p>Diese detaillierte Zeichnung ist ein Hingucker für jedes Zuhause. Die Zeichnung wird in einem hochwertigen, schwarzen Holzrahmen geliefert. Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua.</p>',
                    'ProductVideoProvider' => undef
                }
            }
        },
        '1001-0003' => {
            'Attributes'          => {
                AvailabilityDate                    => undef,
                'EnergyLabelHighestEfficiencyClass' => undef,
                'ExpiryDate'                        => undef,
                'HasSubOwnPrices'                   => '1',
                'RelativeFilePathPart'              => 'Products/1001',
                'PriceQuantity'                     => '1',
                'ImageHotDeal'                      => 'frame_h.jpg',
                'EcoParticipationCategory'          => undef,
                'TaxClass'                          => 'normal',
                'ImageLarge'                        => 'frame.jpg',
                'Manufacturer'                      => undef,
                'EnergyLabelsString'                => undef,
                'OrderUnit'                         => '/Units/piece',
                'RefContentAmount'                  => '1',
                'IsDefault'                         => '0',
                'MinOrder'                          => '1',
                'Position'                          => '30',
                'RefAmount'                         => '1',
                'Height'                            => '3501',
                'IsDailyPrice'                      => '0',
                'RefUnit'                           => '/Units/byte',
                'ImageMedium'                       => 'frame_m.jpg',
                'NewnessDate'                       => undef,
                'RatingAverage'                     => undef,
                'ListPrices'                        => [
                    {
                        'ManufacturerPrice' => '35.01',
                        'CurrencyID'        => 'EUR',
                        'EcoParticipation'  => undef,
                        'ListPrice'         => '38.89',
                        'Deposit'           => undef
                    }
                ],
                'TaxClassDigital'                   => undef,
                'Class'                             => 'MyProductType',
                'DeliveryPeriod'                    => '3',
                'ImageMediumSmall'                  => 'frame_ms.jpg',
                'ImagesSlideShowString'             => 'frame.jpg;frame-1.jpg;frame-2.jpg',
                'IsAvailable'                       => '1',
                'ManufacturerSKU'                   => undef,
                'Width'                             => undef,
                'ImageSmall'                        => 'frame_s.jpg',
                'ImageMediumLarge'                  => 'frame_ml.jpg',
                'IsNew'                             => '0',
                'EnergyLabelSourceFile'             => undef,
                'IsVisible'                         => '1',
                'WeightUnit'                        => '/Units/gram',
                'Weight'                            => '1',
                'UPCEAN'                            => undef,
                'Alias'                             => '1001-0003',
                'Length'                            => undef,
                'MyPreDefLocalizedString'           => 'white',
                'ImageThumbnail'                    => undef,
                'IntervalOrder'                     => '1',
                'SuperProduct'                      => 'Products/1001'
            },
            'LocalizedAttributes' => {
                'en' => {
                    'AvailabilityComment'  => undef,
                    'MainCharacteristics'  => 'Rahmen: Schwarz',
                    'Title'                => undef,
                    'Text'                 => undef,
                    '_ProductDataSheet'    => 'IT_20200422_005933.pdf',
                    'Keywords'             => 'Kunst, Gemälde, Deko',
                    'MyDate'               => '24/07/2020',
                    'MyDateTime'           => '24/07/2020 11:30',
                    'MyFloat'              => '1.5',
                    'MyInteger'            => '11',
                    'MyMoney'              => '5.55 €',
                    'MyString'             => 'someString',
                    'MyTime'               => '12:12',
                    'Name'                 => '[Beispiel] Gerahmte Zeichnung',
                    'Description'          => 'Kurze Beschreibung Rahmen (alt)<p>Diese detaillierte Zeichnung ist ein Hingucker für jedes Zuhause. Die Zeichnung wird in einem hochwertigen, schwarzen Holzrahmen geliefert. Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua.</p>',
                    'ProductVideoProvider' => undef
                }
            }
        },
    },
);

use constant FILENAME => 'products.json';
use constant FILE_DIR => [ qw(ShopData.d export TestShop) ];
use constant FILE_LINES => 5;

sub testExportWithProductsExportImportBase2Prep {
    my $Shop = $TestData{TestShop};

    note('test ProductsExportDriver (Base2Prep)');
    isnt($Shop, undef, 'Testshop exists');
    is($Shop->get('IsMigrationPrepShop'), 0, 'Shop has IsMigrationPrepShop=0');

    my $Exporter = ExportProductsJSON($Shop, $TmpDir);
    ok(ExistsFile($Exporter->fileName), 'export file exists: ' . $Exporter->fileName);
    my $LinesCount = scalar @{GetFileContentLines($Exporter->fileName, { encoding => 'utf8' })};
    is($LinesCount, FILE_LINES, "Export file lines count is ${\FILE_LINES}");
    ok($Exporter->fileName =~ /${\FILENAME}/, "export file name matches ${\FILENAME}");
    my $sFileContent = GetFileContent($Exporter->fileName, { encoding => 'utf8' });
    my $sExpected = GetFileContent(catfile(@{FILE_DIR()}, 'base2prep_' . FILENAME), { encoding => 'utf8' });
    is($$sFileContent, $$sExpected, 'file content matches');

    DeleteFile($Exporter->fileName());
    ok(!ExistsFile($Exporter->fileName()), 'export file cleaned up');
}

sub testExportWithProductsExportImportPrep2Base {
    my $Shop = $TestData{PrepShop};

    note('test ProductsExportDriver (Prep2Base)');
    isnt($Shop, undef, 'PrepShop exists');
    is($Shop->get('IsMigrationPrepShop'), 1, 'Shop has IsMigrationPrepShop=1');

    my $Exporter = ExportProductsJSON($Shop, $TmpDir);
    ok(ExistsFile($Exporter->fileName), 'export file exists: ' . $Exporter->fileName);
    my $LinesCount = scalar @{GetFileContentLines($Exporter->fileName, { encoding => 'utf8' })};
    is($LinesCount, FILE_LINES, "export file lines count is ${\FILE_LINES}");
    ok($Exporter->fileName =~ /${\FILENAME}/, "export file name matches ${\FILENAME}");

    my $sFileContent = GetFileContent($Exporter->fileName, { encoding => 'utf8' });
    my $sExpected = GetFileContent(catfile(@{FILE_DIR()}, 'prep2base_' . FILENAME), { encoding => 'utf8' });
    is($$sFileContent, $$sExpected, 'file content matches');
    DeleteFile($Exporter->fileName());
    ok(!ExistsFile($Exporter->fileName()), 'export file cleaned up');

}

sub testImportWithProductsExportImportBase2Prep {
    my $Shop = $TestData{PrepShop};
    my $DirName = catdir('ShopData.d', 'import');
    note('test ProductsImportHandler (Base2Prep)');
    isnt($Shop, undef, 'Prepshop exists');
    is($Shop->get('IsMigrationPrepShop'), 1, 'Shop has IsMigrationPrepShop=1');
    _compareProducts($Shop, $TestData{OldValues}, 'Old', { FetchPrices => 1 });

    my $ProductFolder = $Shop->folder('Products');
    ok($ProductFolder->existsChild('1002'), 'Product 1002 exists');
    $ProductFolder->child('1002')->delete;
    ok(!$ProductFolder->existsChild('1002'), 'Product 1002 deleted');

    my $Importer = ImportProductsJSON($Shop, $DirName);
    ok($Importer->fileName() =~ /base2prep_${\FILENAME}/, 'ImportFileName was constructed properly');
    ok($ProductFolder->existsChild('1002'), 'Product 1002 was reimported');
    _compareProducts($Shop, $TestData{NewValues}, 'New');
}

sub testImportWithProductsExportImportPrep2Base {
    my $Shop = $TestData{TestShop};
    my $DirName = catdir('ShopData.d', 'import');
    note('test ProductsImportHandler (Prep2Base)');
    isnt($Shop, undef, 'Testshop exists');
    is($Shop->get('IsMigrationPrepShop'), 0, 'Shop has IsMigrationPrepShop=0');
    my $AdaptedValues = getAdapatedValues('OldValues');
    _compareProducts($Shop, $AdaptedValues, 'Old', { FetchText => 1 });

    my $ProductFolder = $Shop->folder('Products');
    ok($ProductFolder->existsChild('1002'), 'Product 1002 exists');
    $ProductFolder->child('1002')->delete;
    ok(!$ProductFolder->existsChild('1002'), 'Product 1002 deleted');

    # ResetProduct converts every Attribute to Type LocalizedStringAttribute
    note('Running ResetProduct()');
    DE_EPAGES::ShopMigration::API::ResetShopHandler::ResetProduct($TestData{TestShop});
    # necessary because we do direct SQL Updates
    ResetFactoryCaches();

    my $Importer = ImportProductsJSON($Shop, $DirName);
    ok($ProductFolder->existsChild('1002'), 'Product 1002 reimported');
    ok($Importer->fileName() =~ /prep2base_${\FILENAME}/, 'ImportFileName was constructed properly');
    $AdaptedValues = getAdapatedValues('NewValues');
    _compareProducts($Shop, $AdaptedValues, 'New', { FetchText => 1 });
}

# We don't want to keep Old and NewValues for both Base and Prepshop therefore we just modify
# the expected values to what is in the base shop at some point (as the expected hash is very long)
sub getAdapatedValues {
    my ($Name) = @_;

    my $AdaptedValues = dclone($TestData{$Name});
    my @DeleteAttr = qw(ProductVideoProvider);
    foreach my $Entry (keys %$AdaptedValues) {
        delete $AdaptedValues->{$Entry}{LocalizedAttributes}{en}{$_} foreach @DeleteAttr;
    }
    my $OldText = '<p>Diese detaillierte Zeichnung ist ein Hingucker für jedes Zuhause. Die Zeichnung wird in einem hochwertigen, schwarzen Holzrahmen geliefert. Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua.</p>';
    my $OldDesc = 'Kurze Beschreibung Rahmen (alt)';
    my $NewText = 'neues Meta';
    my $NewDesc = 'neue Description';
    foreach my $Alias (qw(1001 1001-0001 1001-0002 1001-0003)) {
        if ($Name eq 'OldValues') {
            $AdaptedValues->{$Alias}{LocalizedAttributes}{en}{Text} = $OldText;
            $AdaptedValues->{$Alias}{LocalizedAttributes}{en}{Description} = $OldDesc;

        } else {
            $AdaptedValues->{$Alias}{LocalizedAttributes}{en}{Text} = $NewText;
            $AdaptedValues->{$Alias}{LocalizedAttributes}{en}{Description} = $NewDesc;
        }
    }
    return $AdaptedValues;
}


sub _compareProducts {
    my ($Shop, $TargetValues, $Note, $hOption) = @_;

    my $FetchPrices //= exists $hOption->{FetchPrices} ? $hOption->{FetchPrices} : 0;
    my $FetchText //= exists $hOption->{FetchText} ? $hOption->{FetchText} : 0;
    foreach my $ProductAlias (sort keys %$TargetValues) {
        my $SuperProductPath = $TargetValues->{$ProductAlias}{Attributes}{SuperProduct};
        my $SuperProduct = defined $SuperProductPath ? LoadObjectByPath($SuperProductPath, $Shop->id) : undef;
        my $ProductFolder = defined $SuperProduct ? $SuperProduct->folder('SubProducts') : $Shop->folder('Products');
        my $Product = $ProductFolder->child($ProductAlias);
        my $Values = GetProductData($Shop, $Product);
        delete $Values->{Attributes}{GUID};
        if ($FetchPrices) {
            my $CurrencyID = $Shop->get('CurrencyID');
            my $hEntry = {
                CurrencyID        => $CurrencyID,
                Deposit           => $Product->getDeposit($CurrencyID),
                EcoParticipation  => $Product->getEcoParticipation($CurrencyID),
                ManufacturerPrice => $Product->getManufacturerPrice($CurrencyID),
                ListPrice         => $Product->getListPrice($CurrencyID),
            };
            $Values->{Attributes}{ListPrices} = [ $hEntry ];
        }
        # Text and Description are modifed by GetProductData, so we need to fetch the
        # current values if we have a base shop
        if ($FetchText) {
            my @Override = qw(Text Description Title AvailabilityComment);
            @{$Values->{LocalizedAttributes}{en}}{@Override} =
                @{$Product->getWithParent([ @Override ], GetPKeyLanguageByCode('en'))}{@Override};
            my @ResetOnlyAttributes = qw(IsNew IsDailyPrice PriceQuantity MinOrder IntervalOrder
                IsAvailable AvailabilityDate NewnessDate RatingAverage EcoParticipationCategory ExpiryDate
                ImageMedium ImageMediumLarge ImageMediumSmall ImageSmall ImageThumbnail ImageHotDeal
            );
            @{$Values->{Attributes}}{@ResetOnlyAttributes} =
                @{$Product->getWithParent([ @ResetOnlyAttributes ])}{@ResetOnlyAttributes};
            foreach my $Key (@ResetOnlyAttributes) {
                $Values->{Attributes}{$Key} =
                    defined $Values->{Attributes}{$Key} ?
                        $Values->{Attributes}{$Key} . '' : undef;
            }
        }
        eq_or_diff($Values, $TargetValues->{$ProductAlias},
            "$Note product attribute values for '$ProductAlias' match", { context => 1 });
    }
}

# We need to create a fake ShopTypeRef UnityDE + ECom5000 in case they don't exists
# otherwise the Mapper won't work
sub init {
    my $self = shift;

    $self->SUPER::init;
    my $ShopAlias = $TestData{TestShopAlias};
    my $System = LoadRootObject();
    my $Shop;
    my $TestFolder = $System->folder('Test')->folder('Shops');
    my $ShopTypeRefFolder = $System->folder('ShopTypeRefs');
    my $Class = LoadClassByAlias('ShopTypeRef');
    $Class->insertObject({ Alias => 'UnityDE', 'Parent' => $ShopTypeRefFolder })
        unless ($ShopTypeRefFolder->existsChild('UnityDE'));
    $Class->insertObject({ Alias => 'ECom5000', 'Parent' => $ShopTypeRefFolder })
        unless ($ShopTypeRefFolder->existsChild('ECom5000'));
    if ($TestFolder->existsChild($ShopAlias)) {
        $Shop = $TestFolder->child($ShopAlias);
    } else {
        note(sprintf('Creating %s', $ShopAlias));
        $Shop = $TestFolder->insertChild('Shop', {
            Alias                   => $ShopAlias,
            ShopTypeRef             => $ShopTypeRefFolder->child('ECom5000'),
            WebServerScriptNamePart => $ShopAlias,
        });
        SetShopFeatureValues($Shop,
            [
                { Alias => 'Regions', MaxValue => '2' },
                { Alias => 'Languages', MaxValue => '15' },
                { Alias => 'Products', MaxValue => '100' },
                { Alias => 'ProductTypes', MaxValue => '100' },
                { Alias => 'ProductTypeAttributes', MaxValue => '15' },
                { Alias => 'ProductMaxSubProducts', MaxValue => '50' },
                { Alias => 'Variations', MaxValue => '100' },
            ]);
    }
    $TestData{TestShop} = $Shop;
    $TestData{PrepShop} = ExistsPrepShop($Shop) ? $Shop->get('MigrationPrepShop') : CreatePrepShop($Shop);
    foreach my $JSONFile (@{GetFileNames('ShopData.d', { 'recursive' => 1 })}) {
        Text2Unix($JSONFile) if ($^O eq 'linux');
    }
    ImportObjects('ImportFiles/ProductsBase.xml', $TestData{TestShop}->pathString());
    ImportObjects('ImportFiles/ProductsNow.xml', $TestData{PrepShop}->pathString());
    return;
}

sub cleanup {
    my $self = shift;

    my $ShopAlias = $TestData{TestShopAlias};
    my $TestFolder = LoadRootObject()->folder('Test')->folder('Shops');
    if ($TestFolder->existsChild($ShopAlias)) {
        my $Shop = $TestFolder->child($ShopAlias);
        my $PrepShop = $Shop->get('MigrationPrepShop');
        $Shop->delete() if (defined $Shop);
        $PrepShop->delete();
        LoadRootObject()->folder('Test')->delete();
    }
    DeleteDirectory($TmpDir, { 'recursive' => 1 });
    return;
}

__PACKAGE__->runScript();

1;
