#========================================================================================
# package      DE_EPAGES::ShopMigrationProvider::API::StoreShopMigration
# state        public
#----------------------------------------------------------------------------------------
# description  This package provides helper functions for Shop Migration
#========================================================================================
package DE_EPAGES::ShopMigrationProvider::API::StoreShopMigration;
use base qw( Exporter );

use strict;
our @EXPORT_OK = qw(
    GetStoreShopIDsWithShopTypeChangeAllow
    GetShopRefByStoreIDandStoreObjectID
    ChangeMigrationShopType
    GetMigrationShopTypeMapping
);

use DE_EPAGES::Core::API::Log qw (LogDebug);
use DE_EPAGES::Core::API::TestParameter qw(TestInteger);
use DE_EPAGES::Core::API::Error qw (Error);
use DE_EPAGES::Object::API::Factory qw(LoadObject);
use DE_EPAGES::ShopConfiguration::API::StoreWebServiceClient;
use DE_EPAGES::ShopMigrationProvider::API::Constants qw( SHOPMIGRATION_FINAL_DONE
                                                         %SHOPMIGRATION_SHOPTYPE_MAPPING );
use DE_EPAGES::ShopMigrationProvider::DAL::ShopRef qw (DAL_GetPKeyShopRefsByStoreIDandStoreObjectID);

sub ErrorDefinition {
    return {
        'NO_MAPPING_FOUND' => 'No mapping for shoptype #ShopType found in shopmigration.conf',
    };
}
#========================================================================================
# function     GetStoreShopIDsWithShopTypeChangeAllow
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $aShopIDs = GetStoreShopIDsWithShopTypeChangeAllow( $Store );
#----------------------------------------------------------------------------------------
# description  Gets the list of shopids in the store database
#               having MigrationStatus ShopTypeChangeAllow.
#----------------------------------------------------------------------------------------
# input        $Store | store / database | object
# return       $aShopIDs | array of shopids | ref.array.integer
#========================================================================================
sub GetStoreShopIDsWithShopTypeChangeAllow ($) {
    my ($Store) = @_;

    my $MigrationShopService = DE_EPAGES::ShopConfiguration::API::StoreWebServiceClient->new(
        'store' => $Store,
        'uri' => 'urn://epages.de/WebService/MigrationShopService/2020/11'
    );

    my $aShopIDs = $MigrationShopService->getShopsWithShopTypeChangeAllow()->result;
    LogDebug('aShopIDsSite=', $aShopIDs);
    return $aShopIDs;
}

#========================================================================================
# function     GetShopRefByStoreIDandStoreObjectID
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $ShopRef = GetShopRefByStoreIDandStoreObjectID( $StoreID, $StoreObjectID );
#----------------------------------------------------------------------------------------
# description  Gets the shopref by StoreID and StoreObjectID
#----------------------------------------------------------------------------------------
# input        $StoreID       | storeid       | integer
# input        $StoreObjectID | storeobjectid | integer
# return       $ShopRef       | ShopRef       | object or undef
#========================================================================================
sub GetShopRefByStoreIDandStoreObjectID ($$) {
    my ($StoreID, $StoreObjectID) = @_;
    TestInteger('StoreID', $StoreID);
    TestInteger('StoreObjectID', $StoreID);
    my ($ShopRefID) = @{DAL_GetPKeyShopRefsByStoreIDandStoreObjectID($StoreID, $StoreObjectID)};
    my $ShopRef = defined $ShopRefID? LoadObject($ShopRefID) : undef;
    return $ShopRef;
}

#========================================================================================
# function     ChangeMigrationShopType
# state        public
#----------------------------------------------------------------------------------------
# syntax       ChangeMigrationShopType( $ShopRef );
#----------------------------------------------------------------------------------------
# description  Changes the shoptype of ShopRef to mapped migration shoptype and
#               set MigrationStatus to SHOPMIGRATION_FINAL_DONE.
#----------------------------------------------------------------------------------------
# input        $ShopRef | shopref | object
#========================================================================================
sub ChangeMigrationShopType($) {
    my ($ShopRef) = @_;
    LogDebug('ChangeMigrationShopType for shopref ', $ShopRef->pathString);

    my $ShopType =  $ShopRef->get('ShopType');
    my $IsUnityAlreadyEnabled = $ShopType->get('HasUnityEnabled');

    #set new shoptype
    my $NewShopType = !$IsUnityAlreadyEnabled ? GetMigrationShopTypeMapping($ShopType) : undef;
    $ShopRef->set({'ShopType' => $NewShopType}) if defined $NewShopType;

    #set MigrationStatus to SHOPMIGRATION_FINAL_DONE in Store database
    my $RemoteShop = DE_EPAGES::ShopConfiguration::API::Object::RemoteObject->new(
        Store    => $ShopRef->get('Store'),
        ObjectID => $ShopRef->get('StoreObjectID'),
    );
    $RemoteShop->set({'MigrationStatus' => SHOPMIGRATION_FINAL_DONE});
    LogDebug('ChangeMigrationShopType for shopref done', [$ShopRef->pathString, defined $NewShopType ? $NewShopType->pathString : $ShopType->pathString]);
    return;
}

#========================================================================================
# function     GetMigrationShopTypeMapping
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $MappedShopType = GetMigrationShopTypeMapping( $ShopType );
#----------------------------------------------------------------------------------------
# description  returns the mapped shoptype (defined in shopmigration.conf)
#               for a given ShopType
#----------------------------------------------------------------------------------------
# input        $ShopType | shoptype | object
# return       $MappedShopType | shoptype | object
#========================================================================================
sub GetMigrationShopTypeMapping {
    my ($ShopType) = @_;

    my $ShopTypeAlias = $ShopType->alias;
    my $NewShopTypeAlias;
    if (exists $SHOPMIGRATION_SHOPTYPE_MAPPING{$ShopTypeAlias}) {
        $NewShopTypeAlias = $SHOPMIGRATION_SHOPTYPE_MAPPING{$ShopTypeAlias};
    } else {
        Error('NO_MAPPING_FOUND',{'ShopType' => $ShopTypeAlias});
    }
    return $ShopType->parent->child($NewShopTypeAlias);
}

1;
