use strict;
use Getopt::Long;
use Time::HiRes qw(time);
use DE_EPAGES::Core::API::Error qw(ExistsError GetError);
use DE_EPAGES::Core::API::Warning qw(Warning);
use DE_EPAGES::Core::API::Script qw(RunScript);
use DE_EPAGES::Database::API::Connection qw(RunOnStore);
use DE_EPAGES::Object::API::Factory qw(LoadRootObject);
use DE_EPAGES::ShopConfiguration::API::StoreCartridge qw( IsStoreCartridgeRegistered );
use DE_EPAGES::ShopMigrationProvider::API::StoreShopMigration qw (GetStoreShopIDsWithShopTypeChangeAllow
                                                                  GetShopRefByStoreIDandStoreObjectID
                                                                  ChangeMigrationShopType);
use DE_EPAGES::WebInterface::API::MessageCenter qw(SynchronizeCache);

sub Main {

    local $| = 1;

    my $SitePassword;
    my $StoreName;
    my $SiteName  = 'Site';
    my $Help;
    my $AllStores;
    my $Password;
    my $NoOutput;

    GetOptions(
        'help'          => \$Help,
        'nooutput'      => \$NoOutput,
        'sitename=s'    => \$SiteName,
        'sitepasswd=s'  => \$SitePassword,
        'storename=s'   => \$StoreName,
        'allstores'     => \$AllStores,
        'passwd=s'      => \$Password,
    );

    usage() if $Help;

    if( !defined $StoreName && !$AllStores ) {
        print STDERR "Missing parameter -storename\n\n";
        usage();
    }

    my $time1 = time;

    RunOnStore(
        Store => $SiteName,
        DBPassword => $Password,
        Sub => sub {
            my @StoreNames;
            my $System = LoadRootObject();
            if (defined $AllStores) {
                my $aStores = $System->get('Stores');
                foreach my $Store (@$aStores) {
                    next if $Store->get('Alias') eq $SiteName;
                    $StoreName = $Store->alias;
                    push @StoreNames, $StoreName;
                }
            } else {
                @StoreNames = ($StoreName);
            }
            foreach my $StoreName (@StoreNames) {
                my $Store = $System->child('Stores')->child($StoreName);
                next unless IsStoreCartridgeRegistered( $Store, 'DE_EPAGES::ShopMigration' );

                my $aShopIDs = GetStoreShopIDsWithShopTypeChangeAllow($Store);
                foreach my $StoreObjectID (@$aShopIDs) {
                    my $ShopRef = GetShopRefByStoreIDandStoreObjectID($Store->id, $StoreObjectID);
                    if (defined $ShopRef) {
                        ChangeMigrationShopType($ShopRef);
                    }
                }
            }
        }
    );

    my $time2 = time;
    printf "need %.3f seconds\n", $time2-$time1 unless $NoOutput;

    eval { SynchronizeCache(); };
    Warning( GetError() ) if ExistsError();
    return;
}

sub usage {
    print <<END_USAGE;
Description:
    Updates the shoptypes of shops of the specified database
    which have MigrationStatus=SHOPMIGRATION_FINAL_SHOPTYPECHANGE_ALLOW
Usage:
    perl $0 [options] [flags]

options:
    -sitename   name of site                        (Site)
    -sitepasswd site database user password         ()
    -storename  name of store                       ()
flags:
    -help       show the command line options
    -nooutput   do not print any output

Example:
    perl $0 -allstores -nooutput
    perl $0 -storename Store
END_USAGE
    exit 2;
}

RunScript(
    'Sub' => \&Main
);

1;
