#========================================================================================
# package      DE_EPAGES::Spreedly::UI::PaymentMethodSpreedly
# base         DE_EPAGES::Payment::UI::PaymentMethod
# state        private
#----------------------------------------------------------------------------------------
# description  UI Controller for Test payment method
#========================================================================================
package DE_EPAGES::Spreedly::UI::PaymentMethodSpreedly;
use base DE_EPAGES::Payment::UI::PaymentMethod;

use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Core::API::Log qw ( GetLog );
use DE_EPAGES::ExternalPayment::API::Log qw ( LogPayment );
use DE_EPAGES::ExternalPayment::API::PaymentTypeMethods qw (
    GetPaymentTypeMethodsGroups
    GetSelectedPaymentTypeMethodID
);
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use DE_EPAGES::Spreedly::API::Constants qw (
    IS_PAYMENT_METHOD_OK
    PAYMENT_TYPE_ALIAS_SPREEDLY_TEST
);
use DE_EPAGES::Spreedly::API::GatewayController;
use DE_EPAGES::Spreedly::API::PaymentMethodGenerator;
use DE_EPAGES::Spreedly::API::Tools;
use DE_EPAGES::Spreedly::UI::PaymentMethodSpreedlyStripe;
use DE_EPAGES::Spreedly::UI::PaymentMethodSpreedlyStripePaymentIntents;
use DE_EPAGES::Spreedly::UI::PaymentMethodSpreedlyAuthorize;
use DE_EPAGES::Spreedly::UI::PaymentMethodSpreedlyAuthorizeEvo;
use DE_EPAGES::Spreedly::UI::PaymentMethodSpreedlyMercadoPago;
use DE_EPAGES::Spreedly::UI::PaymentMethodSpreedlyPayULatam;
use DE_EPAGES::Spreedly::UI::PaymentMethodSpreedlyPayeezy;

use strict;

#========================================================================================
# function     ViewSettings
# state        public
#----------------------------------------------------------------------------------------
# syntax       ViewSettings( $Servlet );
#----------------------------------------------------------------------------------------
# description  UI Controller to view settings on the MBO settings page.
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
#========================================================================================
sub ViewSettings {
    my $self = shift;
    my ($Servlet) = @_;

    my $PaymentMethod = $Servlet->object();

    $Servlet->vars(
        'PaymentTypeMethodsGroups',
        GetPaymentTypeMethodsGroups( $PaymentMethod->get( 'PaymentType' ), $Servlet->languageID )
    );
    $Servlet->vars(
        'PaymentTypeMethod',
        GetSelectedPaymentTypeMethodID( $PaymentMethod )
    );

    GetLog()->info('PaymentType name: ' . $PaymentMethod->get('PaymentType')->alias());

    if ($PaymentMethod->get('PaymentType')->alias() eq 'SpreedlyStripe') {
        $Servlet->vars('PaymentName', 'Stripe');
        $Servlet = $self->_buildViewSettingsStripe($Servlet, $PaymentMethod);
    }
    elsif ($PaymentMethod->get('PaymentType')->alias() eq 'SpreedlyStripePaymentIntents') {
        $Servlet->vars('PaymentName', 'StripePaymentIntents');
        $Servlet = $self->_buildViewSettingsStripePaymentIntents($Servlet, $PaymentMethod);
    }
    elsif ($PaymentMethod->get('PaymentType')->alias() eq 'SpreedlyAuthorize'){
        $Servlet->vars('PaymentName', 'Authorize.Net');
        $Servlet = $self->_buildViewSettingsAuthorize($Servlet, $PaymentMethod);
    }
    elsif ($PaymentMethod->get('PaymentType')->alias() eq 'SpreedlyAuthorizeEvo'){
        $Servlet->vars('PaymentName', 'EVO');
        $Servlet = $self->_buildViewSettingsAuthorizeEvo($Servlet, $PaymentMethod);
    }
    elsif ($PaymentMethod->get('PaymentType')->alias() eq 'SpreedlyMercadoPago'){
        $Servlet->vars('PaymentName', 'MercadoPago');
        $Servlet = $self->_buildViewSettingsMercadoPago($Servlet, $PaymentMethod);
    }
    elsif ($PaymentMethod->get('PaymentType')->alias() eq 'SpreedlyPayULatam'){
        $Servlet->vars('PaymentName', 'PayULatam');
        $Servlet = $self->_buildViewSettingsPayULatam($Servlet, $PaymentMethod);
    }
    elsif ($PaymentMethod->get('PaymentType')->alias() eq 'SpreedlyPayeezy'){
        $Servlet->vars('PaymentName', 'Payeezy');
        $Servlet = $self->_buildViewSettingsPayeezy($Servlet, $PaymentMethod);
    } else {
        $Servlet->vars('PaymentName', 'SpreedlyTest');
    }

    my $Shop = $PaymentMethod->getSite();
    my $hCredentials = $PaymentMethod->getGatewayCredentials($Shop);
    $Servlet->vars('IsTokenSet', isValid($hCredentials->{'token'}));

    return;
}



#========================================================================================
# function     SaveSettings
# state        public
#----------------------------------------------------------------------------------------
# syntax       SaveSettings( $Servlet );
#----------------------------------------------------------------------------------------
# description  UI Controller to save settings on the MBO settings page.
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
#========================================================================================
sub SaveSettings {
    my $self = shift;
    my ($Servlet) = @_;

    my $PaymentMethod = $Servlet->object();
    my $PaymentType = $PaymentMethod->get('PaymentType');
    my $PaymentTypeAlias = $PaymentType->alias();
    if ($PaymentTypeAlias eq PAYMENT_TYPE_ALIAS_SPREEDLY_TEST) {
        $self->SaveCredentials($Servlet, $PaymentTypeAlias);
    }

    $self->_saveLikePaymentTypeMethod($Servlet, $PaymentMethod);

    $self->Save($Servlet);

    return;
}

#========================================================================================
# function     NewSpreedlyMethod
# state        public
#----------------------------------------------------------------------------------------
# syntax       NewSpreedlyMethod( $Servlet );
#----------------------------------------------------------------------------------------
# description  Creates new payment methods with the existing Spreedly credentials
#----------------------------------------------------------------------------------------
# input        $Servlet | Input1Description | ref_hash
#========================================================================================
sub NewSpreedlyMethod {
    my $self = shift;
    my ($Servlet) = @_;

    my $PaymentMethod = $Servlet->object();
    my $Values = $Servlet->form()->formValues();

    my $aSelectedPaymentMethods = $Values->{'PaymentMethodToGenerateID'};
    my $NumberOfSelectedPaymentMethods;
    if (isValid($aSelectedPaymentMethods)) {
        $NumberOfSelectedPaymentMethods = scalar @{ $aSelectedPaymentMethods };
    } else {
        $NumberOfSelectedPaymentMethods = 0;
    }

    if($NumberOfSelectedPaymentMethods > 0) {

        my $PaymentMethodGenerator = DE_EPAGES::Spreedly::API::PaymentMethodGenerator->new();
        my $Status = $PaymentMethodGenerator->generatePaymentMethods($PaymentMethod, $aSelectedPaymentMethods);

        if ($Status == IS_PAYMENT_METHOD_OK) {
            my $NewPaymentMethods = $PaymentMethodGenerator->getGeneratedPaymentMethods();
            $Servlet->vars('NewPaymentMethods',$NewPaymentMethods);
        } else {
            $Servlet->vars('NewSpreedlyMethodError', 1);
        }

    } else {
        $Servlet->vars('NewSpreedlyMethodNothingSelected', 1);
    }

    return;
}

#========================================================================================
# function     SaveCredentials
# state        public
#----------------------------------------------------------------------------------------
# syntax       SaveCredentials( $Servlet, $PaymentTypeAlias, $hAdditionalBodyParams );
#----------------------------------------------------------------------------------------
# description  It launches gateway credentials saving through epagesj and collects response
#----------------------------------------------------------------------------------------
# input        $Servlet | Input1Description | ref_hash
# input        $PaymentTypeAlias | PaymentType name | string
# input        $hAdditionalBodyParams | hash with request parameters | ref_hash
# return       $Response | hash with epagesj gateway endpoint response | ref_hash
#========================================================================================
sub SaveCredentials {
    my $self = shift;
    my ($Servlet, $PaymentTypeAlias, $hAdditionalBodyParams) = @_;

    my $GatewayController = DE_EPAGES::Spreedly::API::GatewayController->new($Servlet);

    my $hResponse;
    eval {
        $hResponse = $GatewayController->saveCredentials($hAdditionalBodyParams);
    };

    if ( ExistsError() ) {
        my $Error = GetError();
        LogPayment($PaymentTypeAlias,'CheckSpreedlyConnection' , $Error->longMessage(), 'Error');
        my $Form = $Servlet->form();
        if (isValid($Form->formName())) {
            $Form->executeFormError( {
                    'Name'       => 'GatewayCredentials',
                    'Reason'     => 'SpreedlyConnectionError'
                }
            );
        }
        else {
            $Servlet->vars('SpreedlyConnectionError', 1);
        }
    }

    return $hResponse;
}

#----------------------------------------------------------------------------------------
# Private methods
#----------------------------------------------------------------------------------------
sub _buildViewSettingsStripe {
    my $self = shift;
    my ($Servlet, $PaymentMethod) = @_;

    my $PaymentMethodSpreedlyStripe = DE_EPAGES::Spreedly::UI::PaymentMethodSpreedlyStripe->new($Servlet, $PaymentMethod);

    return $PaymentMethodSpreedlyStripe->ViewSettingsSpreedlyStripe();
}

sub _buildViewSettingsStripePaymentIntents {
    my $self = shift;
    my ($Servlet, $PaymentMethod) = @_;

    my $PaymentMethodSpreedlyStripePaymentIntents =
        DE_EPAGES::Spreedly::UI::PaymentMethodSpreedlyStripePaymentIntents->new($Servlet, $PaymentMethod);

    return $PaymentMethodSpreedlyStripePaymentIntents->ViewSettingsSpreedlyStripePaymentIntents();
}

sub _buildViewSettingsAuthorize {
    my $self = shift;
    my ($Servlet, $PaymentMethod) = @_;

    return DE_EPAGES::Spreedly::UI::PaymentMethodSpreedlyAuthorize->ViewSettingsSpreedlyAuthorize($Servlet, $PaymentMethod);
}

sub _buildViewSettingsAuthorizeEvo {
    my $self = shift;
    my ($Servlet, $PaymentMethod) = @_;

    return DE_EPAGES::Spreedly::UI::PaymentMethodSpreedlyAuthorizeEvo->ViewSettingsSpreedlyAuthorizeEvo($Servlet, $PaymentMethod);
}

sub _buildViewSettingsMercadoPago {
    my $self = shift;
    my ($Servlet, $PaymentMethod) = @_;

    return DE_EPAGES::Spreedly::UI::PaymentMethodSpreedlyMercadoPago->ViewSettingsSpreedlyMercadoPago($Servlet, $PaymentMethod);
}

sub _buildViewSettingsPayULatam {
    my $self = shift;
    my ($Servlet, $PaymentMethod) = @_;

    return DE_EPAGES::Spreedly::UI::PaymentMethodSpreedlyPayULatam->ViewSettingsSpreedlyPayULatam($Servlet, $PaymentMethod);
}

sub _buildViewSettingsPayeezy {
    my $self = shift;
    my ($Servlet, $PaymentMethod) = @_;

    return DE_EPAGES::Spreedly::UI::PaymentMethodSpreedlyPayeezy->ViewSettingsSpreedlyPayeezy($Servlet, $PaymentMethod);
}

sub _saveLikePaymentTypeMethod {
    my $self = shift;
    my ($Servlet, $PaymentMethod) = @_;

    my $CurrentPaymentTypeMethodID = 0;
    my $CurrentPaymentTypeMetod = $PaymentMethod->get('PaymentTypeMethod');
    $CurrentPaymentTypeMethodID = $CurrentPaymentTypeMetod->id() if (defined($CurrentPaymentTypeMetod));

    my $Form = $Servlet->form();
    my $NextPaymentTypeMethodID = $Form->value('PaymentTypeMethod');

    if ($CurrentPaymentTypeMethodID != $NextPaymentTypeMethodID) {
        my $NextPaymentTypeMethod = LoadRootObject()->loadObject($NextPaymentTypeMethodID, 'PaymentTypeMethod');

        $PaymentMethod->setInShopNameLikePaymentTypeMethod($NextPaymentTypeMethod);
        $PaymentMethod->setPaymentMethodNameLikePaymentTypeMethod($NextPaymentTypeMethod, $Servlet->languageID);
        $PaymentMethod->uploadPaymentTypeMethodLogo($NextPaymentTypeMethod);
    }

}

1;
