use strict;

use Getopt::Long;
use Time::HiRes qw (time);

use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( GetCurrentDBHandle RunOnStore Transaction );
use DE_EPAGES::DataCache::API::Cache qw( UpdateConnection);
use DE_EPAGES::Object::API::Factory qw( LoadRootObject );
use DE_EPAGES::Core::API::Error qw( GetError ExistsError);
use DE_EPAGES::WebInterface::API::MessageCenter qw( SynchronizeCache );
use DE_EPAGES::Core::API::Warning qw( Warning );

RunScript( Sub => \&main );

#
#   What it does:
#   Backup (move) the given tax matrix to a backup folder and clone
#   the backed up matrix to the original place
#   Then the shops using this tax matrix are set to the cloned tax matrix
#   This handling is used for having the old tax matrix on orders
#   without destroing the taxes on tax changes

#========================================================================================
# main function
#========================================================================================
sub main {
    my ( $Help, $StoreName, $TaxMatrixAlias, $ObsoleteDate );

    local $| = 1;

    GetOptions(
        'help'              => \$Help,
        'storename=s'       => \$StoreName,
        'taxmatrixalias=s'  => \$TaxMatrixAlias,
        'validuntil=s'      => \$ObsoleteDate
    );

    usage() if $Help;

    unless ( $StoreName and $TaxMatrixAlias and $ObsoleteDate ) {
        print STDERR "Missing parameter\n\n";
        usage();
    }

    my $time1 = time();

    RunOnStore(
        Store => $StoreName,
        Sub   => sub { BackupTaxMatrix( $TaxMatrixAlias, $ObsoleteDate ) }
    );

    my $time2   = time;
    my $seconds = $time2 - $time1;
    printf "\nneed %.3f seconds over all\n", $seconds;
}

#========================================================================================

sub usage {
    print <<END_USAGE;
Note:
   Backup (move with the same objectid) the given tax matrix to a backup folder and clone
   the backed up matrix to the original place
   Then the shops tax matrix using this tax matrix are set to the cloned tax matrix
   This handling is used for having the old tax matrix on orders
   for allowing changes on the order by using the old taxes
   IMPORT NOTICE:
   If you have problems during runtime of the script, fix the problems and
   start it with the same parameters again. You need at least one sucessfull complete run.

Usage:
    perl $0 [options]

options:
    -storename     store name for the database connection
    -taxmatrixalias alias of the TaxMatrix to back up
    -validuntil    date of invalidation of tax matrix

Example:
    perl $0 -storename Store -taxmatrixalias TaxMatrixGermany -validuntil 28082020
END_USAGE

    exit(0);
}

#========================================================================================

sub BackupTaxMatrix {
    my ( $TaxMatrixToBackupAlias, $ObsoleteDate ) = @_;
    my $System = LoadRootObject();

    my $TaxMatrixToBackup;
    my $NewTaxMatrixFromBackup;

    unless ($System->existsChild($TaxMatrixToBackupAlias)) {
        print "TaxMatrix to backup: $TaxMatrixToBackupAlias does not exists.\n";
        return;
    } else {
        $TaxMatrixToBackup = $System->child($TaxMatrixToBackupAlias);
        my $BackupFolder = $System->folder('TaxMatrixBackup');

        # set alias for new backuped taxmatrix
        my $TaxMatrixToBackupBackupAlias = $TaxMatrixToBackupAlias . '_' . $ObsoleteDate;

        # use already saved Backuptaxmatrix or create/clone new backup
        if ($BackupFolder->existsChild($TaxMatrixToBackupBackupAlias)) {
            print "Backup TaxMatrix with the name: $TaxMatrixToBackupBackupAlias already exists in backup folder.\n";
            $NewTaxMatrixFromBackup = $TaxMatrixToBackup;
        } else {
            Transaction( sub {
                # move matrix to backup folder
                print "Move TaxMatrix: $TaxMatrixToBackupAlias as $TaxMatrixToBackupBackupAlias to backup folder: /TaxMatrixBackup \n";
                $TaxMatrixToBackup->set({
                    'Parent' => $BackupFolder,
                    'Alias'  => $TaxMatrixToBackupBackupAlias
                });

                # clone Matrix back with old alias
                print "Clone moved TaxMatrix: $TaxMatrixToBackupBackupAlias to /$TaxMatrixToBackupAlias\n";
                $NewTaxMatrixFromBackup = $TaxMatrixToBackup->clone({
                    'Alias' => $TaxMatrixToBackupAlias,
                    'Parent'=> $System
                },1);

                # set system tax matrix on system if the changed TaxMatrix was on the System as TaxMatrix
                if ($System->get('TaxMatrix')->id eq $TaxMatrixToBackup->id){
                    print "Set cloned TaxMatrix: $TaxMatrixToBackupAlias on System attribute TaxMatrix\n";
                    $System->set({'TaxMatrix' => $NewTaxMatrixFromBackup });
                }
            });

            # prevent caching problems
            eval {
                UpdateConnection( GetCurrentDBHandle()->cacheKey );
                SynchronizeCache();
            };
            Warning( GetError() ) if ExistsError();
        }
    }

    # set shop to copied new matrix (this matrix can then hold the tax new values)
    my $TaxMatrixToBackupID = $TaxMatrixToBackup->id;
    eval {
        foreach my $Shop (@{$System->get('Shops')}) {
            # next if shop already has new backup taxmatrix set
            next if $Shop->get('TaxMatrix')->id eq $NewTaxMatrixFromBackup->id;
            if ($Shop->get('TaxMatrix')->id eq $TaxMatrixToBackupID) {
                print "set Shop: " . $Shop->alias . " to new TaxMatrix: $TaxMatrixToBackupAlias\n";
                $Shop->set({ 'TaxMatrix' => $NewTaxMatrixFromBackup });
            }
        }
    };
    Warning( GetError() ) if ExistsError();

    # flush outdated cacheinfo
    print "Clear ShopCache...\n";

    eval {
        UpdateConnection( GetCurrentDBHandle()->cacheKey );
        SynchronizeCache();
    };
    Warning( GetError() ) if ExistsError();
    print "\n**********Now you can change the TaxMatrix xml file for: $TaxMatrixToBackupAlias and import the file.**********\n";

}
